@@ -1,13 +1,13 @@
 /*
  * difffile.c - DIFF file handling source code. Read and write diff files.
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
  */
 
-#include "config.h"
+#include <config.h>
 #include <assert.h>
 #include <string.h>
 #include <unistd.h>
@@ -17,53 +17,45 @@
 #include "util.h"
 #include "packet.h"
 #include "rdata.h"
-#include "nsec3.h"
 
-static int
+static int 
 write_32(FILE *out, uint32_t val)
 {
 	val = htonl(val);
 	return write_data(out, &val, sizeof(val));
 }
 
-static int
+static int 
 write_16(FILE *out, uint16_t val)
 {
 	val = htons(val);
 	return write_data(out, &val, sizeof(val));
 }
 
-static int
+static int 
 write_8(FILE *out, uint8_t val)
 {
 	return write_data(out, &val, sizeof(val));
 }
 
-static int
+static int 
 write_str(FILE *out, const char* str)
 {
 	uint32_t len = strlen(str);
-	if(!write_32(out, len))
+	if(!write_32(out, len)) 
 		return 0;
 	return write_data(out, str, len);
 }
 
-void
-diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
+void 
+diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id, 
 	uint32_t seq_nr, uint8_t* data, size_t len, nsd_options_t* opt)
 {
 	const char* filename = opt->difffile;
-	struct timeval tv;
 	FILE *df;
-	uint32_t file_len = sizeof(uint32_t) + strlen(zone) +
+	uint32_t file_len = sizeof(uint32_t) + strlen(zone) + 
 		sizeof(new_serial) + sizeof(id) + sizeof(seq_nr) + len;
 
-	if (gettimeofday(&tv, NULL) != 0) {
-		log_msg(LOG_ERR, "could not set timestamp for %s: %s",
-			filename, strerror(errno));
-		return;
-	}
-
 	df = fopen(filename, "a");
 	if(!df) {
 		log_msg(LOG_ERR, "could not open file %s for append: %s",
@@ -72,61 +64,13 @@ diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
 	}
 
 	if(!write_32(df, DIFF_PART_IXFR) ||
-		!write_32(df, (uint32_t) tv.tv_sec) ||
-		!write_32(df, (uint32_t) tv.tv_usec) ||
 		!write_32(df, file_len) ||
 		!write_str(df, zone) ||
 		!write_32(df, new_serial) ||
 		!write_16(df, id) ||
 		!write_32(df, seq_nr) ||
 		!write_data(df, data, len) ||
-		!write_32(df, file_len))
-	{
-		log_msg(LOG_ERR, "could not write to file %s: %s",
-			filename, strerror(errno));
-	}
-	fclose(df);
-}
-
-void
-diff_write_commit(const char* zone, uint32_t old_serial,
-	uint32_t new_serial, uint16_t id, uint32_t num_parts,
-	uint8_t commit, const char* log_str, nsd_options_t* opt)
-{
-	const char* filename = opt->difffile;
-	struct timeval tv;
-	FILE *df;
-	uint32_t len;
-
-	if (gettimeofday(&tv, NULL) != 0) {
-		log_msg(LOG_ERR, "could not set timestamp for %s: %s",
-			filename, strerror(errno));
-		return;
-	}
-
-	df = fopen(filename, "a");
-	if(!df) {
-		log_msg(LOG_ERR, "could not open file %s for append: %s",
-			filename, strerror(errno));
-		return;
-	}
-
-	len = strlen(zone) + sizeof(len) + sizeof(old_serial) +
-		sizeof(new_serial) + sizeof(id) + sizeof(num_parts) +
-		sizeof(commit) + strlen(log_str) + sizeof(len);
-
-	if(!write_32(df, DIFF_PART_SURE) ||
-		!write_32(df, (uint32_t) tv.tv_sec) ||
-		!write_32(df, (uint32_t) tv.tv_usec) ||
-		!write_32(df, len) ||
-		!write_str(df, zone) ||
-		!write_32(df, old_serial) ||
-		!write_32(df, new_serial) ||
-		!write_16(df, id) ||
-		!write_32(df, num_parts) ||
-		!write_8(df, commit) ||
-		!write_str(df, log_str) ||
-		!write_32(df, len))
+		!write_32(df, file_len)) 
 	{
 		log_msg(LOG_ERR, "could not write to file %s: %s",
 			filename, strerror(errno));
@@ -135,11 +79,46 @@ diff_write_commit(const char* zone, uint32_t old_serial,
 	fclose(df);
 }
 
-/*
- * Checksum to signal no data change occured (for example, by a
- * zonec run.
- */
-int
+void 
+diff_write_commit(const char* zone, uint32_t old_serial,
+	uint32_t new_serial, uint16_t id, uint32_t num_parts,
+	uint8_t commit, const char* log_str, 
+	nsd_options_t* opt)
+{
+	const char* filename = opt->difffile;
+	FILE *df;
+	uint32_t len;
+
+	df = fopen(filename, "a");
+	if(!df) {
+		log_msg(LOG_ERR, "could not open file %s for append: %s",
+			filename, strerror(errno));
+		return;
+	}
+
+	len = strlen(zone)+sizeof(len) + sizeof(old_serial) + 
+		sizeof(new_serial) + sizeof(id) + sizeof(num_parts) +
+		sizeof(commit) + strlen(log_str)+sizeof(len);
+
+	if(!write_32(df, DIFF_PART_SURE) ||
+		!write_32(df, len) ||
+		!write_str(df, zone) ||
+		!write_32(df, old_serial) ||
+		!write_32(df, new_serial) ||
+		!write_16(df, id) ||
+		!write_32(df, num_parts) ||
+		!write_8(df, commit) ||
+		!write_str(df, log_str) ||
+		!write_32(df, len)) 
+	{
+		log_msg(LOG_ERR, "could not write to file %s: %s",
+			filename, strerror(errno));
+	}
+	fflush(df);
+	fclose(df);
+}
+
+int 
 db_crc_different(namedb_type* db)
 {
 	FILE *fd = fopen(db->filename, "r");
@@ -150,7 +129,7 @@ db_crc_different(namedb_type* db)
 			db->filename, strerror(errno));
 		return -1;
 	}
-
+	
 	/* seek to position of CRC, check it and magic no */
 	if(fseeko(fd, db->crc_pos, SEEK_SET)==-1) {
 		log_msg(LOG_ERR, "unable to fseeko %s: %s. db changed?",
@@ -187,18 +166,18 @@ db_crc_different(namedb_type* db)
 	return 1;
 }
 
-int
+int 
 diff_read_32(FILE *in, uint32_t* result)
 {
-	if (fread(result, sizeof(*result), 1, in) == 1) {
-		*result = ntohl(*result);
-		return 1;
-	} else {
-		return 0;
-	}
+        if (fread(result, sizeof(*result), 1, in) == 1) {
+                *result = ntohl(*result);
+                return 1;
+        } else {
+                return 0;
+        }
 }
 
-int
+int 
 diff_read_16(FILE *in, uint16_t* result)
 {
         if (fread(result, sizeof(*result), 1, in) == 1) {
@@ -209,7 +188,7 @@ diff_read_16(FILE *in, uint16_t* result)
         }
 }
 
-int
+int 
 diff_read_8(FILE *in, uint8_t* result)
 {
         if (fread(result, sizeof(*result), 1, in) == 1) {
@@ -219,15 +198,15 @@ diff_read_8(FILE *in, uint8_t* result)
         }
 }
 
-int
+int 
 diff_read_str(FILE* in, char* buf, size_t len)
 {
 	uint32_t disklen;
-	if(!diff_read_32(in, &disklen))
+	if(!diff_read_32(in, &disklen)) 
 		return 0;
-	if(disklen >= len)
+	if(disklen >= len) 
 		return 0;
-	if(fread(buf, disklen, 1, in) != 1)
+	if(fread(buf, disklen, 1, in) != 1) 
 		return 0;
 	buf[disklen] = 0;
 	return 1;
@@ -242,15 +221,15 @@ add_rdata_to_recyclebin(namedb_type* db, rr_type* rr)
 	{
 		if(!rdata_atom_is_domain(rr->type, i))
 			region_recycle(db->region, rr->rdatas[i].data,
-				rdata_atom_size(rr->rdatas[i])
+				rdata_atom_size(rr->rdatas[i]) 
 				+ sizeof(uint16_t));
 	}
-	region_recycle(db->region, rr->rdatas,
+	region_recycle(db->region, rr->rdatas, 
 		sizeof(rdata_atom_type)*rr->rdata_count);
 }
 
 /* this routine determines if below a domain there exist names with
- * data (is_existing) or no names below the domain have data.
+ * data (is_existing) or no names below the domain have data. 
  */
 static int
 has_data_below(domain_type* top)
@@ -267,35 +246,7 @@ has_data_below(domain_type* top)
 	return 0;
 }
 
-
-/* this routine makes empty terminals non-existent.
- * @domain the lowest empty terminal
- * @ce the closest encloser
- */
-static domain_type*
-rrset_delete_empty_terminals(domain_type* domain, domain_type* ce)
-{
-	assert(domain);
-	if (domain->rrsets == 0) {
-		/* if there is no data below it, it becomes non existing.
-		   also empty nonterminals above it become nonexisting */
-		/* check for data below this node. */
-		if(!has_data_below(domain)) {
-			/* nonexist this domain and all parent empty nonterminals */
-			domain_type* p = domain;
-			while(p != NULL && p->rrsets == 0) {
-				if(p == ce || has_data_below(p))
-					return p;
-				p->is_existing = 0;
-				p = p->parent;
-			}
-		}
-	}
-	return NULL;
-}
-
-
-static domain_type*
+static void 
 rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 {
 	int i;
@@ -306,11 +257,11 @@ rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 	}
 	if(!*pp) {
 		/* rrset does not exist for domain */
-		return NULL;
+		return;
 	}
 	*pp = rrset->next;
 
-	DEBUG(DEBUG_XFRD,2, (LOG_INFO, "delete rrset of %s type %s",
+	DEBUG(DEBUG_XFRD,2, (LOG_INFO, "delete rrset of %s type %s", 
 		dname_to_string(domain_dname(domain),0),
 		rrtype_to_string(rrset_rrtype(rrset))));
 
@@ -318,56 +269,58 @@ rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 	if(rrset->zone->soa_rrset == rrset) {
 		rrset->zone->soa_rrset = 0;
 		rrset->zone->updated = 1;
-		domain->has_SOA = 0;
 	}
 	if(rrset->zone->ns_rrset == rrset) {
 		rrset->zone->ns_rrset = 0;
 	}
+#ifdef DNSSEC
 	if(domain == rrset->zone->apex && rrset_rrtype(rrset) == TYPE_RRSIG) {
 		for (i = 0; i < rrset->rr_count; ++i) {
-			if (rr_rrsig_type_covered(&rrset->rrs[i]) == TYPE_DNSKEY) {
+			if (rr_rrsig_type_covered(&rrset->rrs[i]) == TYPE_SOA) {
 				rrset->zone->is_secure = 0;
 				break;
 			}
 		}
 	}
-
-#ifdef NSEC3
-#ifndef FULL_PREHASH
-	if (rrset->rrs[0].type == TYPE_NSEC3) {
-		namedb_del_nsec3_domain(db, domain, rrset->zone);
-	}
-#endif /* !FULL_PREHASH */
-#endif /* NSEC3 */
-
+#endif
 	/* recycle the memory space of the rrset */
 	for (i = 0; i < rrset->rr_count; ++i)
 		add_rdata_to_recyclebin(db, &rrset->rrs[i]);
-	region_recycle(db->region, rrset->rrs,
+	region_recycle(db->region, rrset->rrs, 
 		sizeof(rr_type) * rrset->rr_count);
-	rrset->rr_count = 0;
 	region_recycle(db->region, rrset, sizeof(rrset_type));
 
 	/* is the node now an empty node (completely deleted) */
-	if (domain->rrsets == 0) {
-		return domain;
+	if(domain->rrsets == 0) {
+		/* if there is no data below it, it becomes non existing.
+		   also empty nonterminals above it become nonexisting */
+		/* check for data below this node. */
+		if(!has_data_below(domain)) {
+			/* nonexist this domain and all parent empty nonterminals */
+			domain_type* p = domain;
+			while(p != NULL && p->rrsets == 0) {
+				p->is_existing = 0;
+				p = p->parent;
+			}
+		}
 	}
-	return NULL;
+	rrset->rr_count = 0;
 }
 
-static int
+static int 
 rdatas_equal(rdata_atom_type *a, rdata_atom_type *b, int num, uint16_t type)
 {
 	int k;
 	for(k = 0; k < num; k++)
 	{
 		if(rdata_atom_is_domain(type, k)) {
+			/* check dname */
 			if(dname_compare(domain_dname(a[k].domain),
 				domain_dname(b[k].domain))!=0)
 				return 0;
 		} else {
 			/* check length */
-			if(a[k].data[0] != b[k].data[0])
+			if(a[k].data[0] != b[k].data[0]) 
 				return 0;
 			/* check data */
 			if(memcmp(a[k].data+1, b[k].data+1, a[k].data[0])!=0)
@@ -377,45 +330,41 @@ rdatas_equal(rdata_atom_type *a, rdata_atom_type *b, int num, uint16_t type)
 	return 1;
 }
 
-static int
+static int 
 find_rr_num(rrset_type* rrset,
-	uint16_t type, uint16_t klass,
+	uint16_t type, uint16_t klass, uint32_t ttl, 
 	rdata_atom_type *rdatas, ssize_t rdata_num)
 {
 	int i;
-
-	for(i=0; i < rrset->rr_count; ++i) {
-		if(rrset->rrs[i].type == type &&
+	for(i=0; i<rrset->rr_count; ++i) {
+		if(rrset->rrs[i].ttl == ttl &&
+		   rrset->rrs[i].type == type &&
 		   rrset->rrs[i].klass == klass &&
 		   rrset->rrs[i].rdata_count == rdata_num &&
 		   rdatas_equal(rdatas, rrset->rrs[i].rdatas, rdata_num, type))
-		{
 			return i;
-		}
 	}
-
 	return -1;
 }
 
-static int
-delete_RR(namedb_type* db, const dname_type* dname,
-	uint16_t type, uint16_t klass,
-	domain_type* prevdomain,
+static int 
+delete_RR(namedb_type* db, const dname_type* dname, 
+	uint16_t type, uint16_t klass, uint32_t ttl, 
 	buffer_type* packet, size_t rdatalen, zone_type *zone,
-	region_type* temp_region, int is_axfr)
+	region_type* temp_region)
 {
 	domain_type *domain;
 	rrset_type *rrset;
 	domain = domain_table_find(db->domains, dname);
 	if(!domain) {
-		log_msg(LOG_WARNING, "diff: domain %s does not exist",
+		log_msg(LOG_WARNING, "diff: domain %s does not exist", 
 			dname_to_string(dname,0));
 		buffer_skip(packet, rdatalen);
 		return 1; /* not fatal error */
 	}
 	rrset = domain_find_rrset(domain, zone, type);
 	if(!rrset) {
-		log_msg(LOG_WARNING, "diff: rrset %s does not exist",
+		log_msg(LOG_WARNING, "diff: rrset %s does not exist", 
 			dname_to_string(dname,0));
 		buffer_skip(packet, rdatalen);
 		return 1; /* not fatal error */
@@ -426,46 +375,22 @@ delete_RR(namedb_type* db, const dname_type* dname,
 		ssize_t rdata_num;
 		int rrnum;
 		temptable = domain_table_create(temp_region);
-		/* This will ensure that the dnames in rdata are
-		 * normalized, conform RFC 4035, section 6.2
-		 */
 		rdata_num = rdata_wireformat_to_rdata_atoms(
 			temp_region, temptable, type, rdatalen, packet, &rdatas);
 		if(rdata_num == -1) {
-			log_msg(LOG_ERR, "diff: bad rdata for %s",
+			log_msg(LOG_ERR, "diff: bad rdata for %s", 
 				dname_to_string(dname,0));
 			return 0;
 		}
-		rrnum = find_rr_num(rrset, type, klass, rdatas, rdata_num);
+		rrnum = find_rr_num(rrset, type, klass, ttl, rdatas, rdata_num);
 		if(rrnum == -1) {
-			log_msg(LOG_WARNING, "diff: RR <%s, %s> does not exist",
-				dname_to_string(dname,0), rrtype_to_string(type));
+			log_msg(LOG_WARNING, "diff: RR %s does not exist", 
+				dname_to_string(dname,0));
 			return 1; /* not fatal error */
 		}
-#ifdef NSEC3
-#ifndef FULL_PREHASH
-		if (is_axfr == 0) {
-			struct domain *parent = domain;
-			do {
-				if (0 != namedb_add_nsec3_mod_domain(db,
-								    parent)) {
-					return 0;
-				}
-				parent = parent->parent;
-			} while (parent != zone->apex->parent);
-		}
-#else
-		(void)is_axfr;
-#endif /* !FULL_PREHASH */
-#endif /* NSEC3 */
-
 		if(rrset->rr_count == 1) {
 			/* delete entire rrset */
-			domain = rrset_delete(db, domain, rrset);
-			if (domain && domain != prevdomain && !domain->nextdiff) {
-				/* this domain is not yet in the diff chain */
-				prevdomain->nextdiff = domain;
-			}
+			rrset_delete(db, domain, rrset);
 		} else {
 			/* swap out the bad RR and decrease the count */
 			rr_type* rrs_orig = rrset->rrs;
@@ -488,11 +413,10 @@ delete_RR(namedb_type* db, const dname_type* dname,
 	return 1;
 }
 
-static int
-add_RR(namedb_type* db, const dname_type* dname,
-	uint16_t type, uint16_t klass, uint32_t ttl,
-	buffer_type* packet, size_t rdatalen, zone_type *zone,
-	int is_axfr)
+static int 
+add_RR(namedb_type* db, const dname_type* dname, 
+	uint16_t type, uint16_t klass, uint32_t ttl, 
+	buffer_type* packet, size_t rdatalen, zone_type *zone)
 {
 	domain_type* domain;
 	rrset_type* rrset;
@@ -519,27 +443,24 @@ add_RR(namedb_type* db, const dname_type* dname,
 		domain_add_rrset(domain, rrset);
 	}
 
-	/* dnames in rdata are normalized, conform RFC 4035,
-	 * Section 6.2
-	 */
 	rdata_num = rdata_wireformat_to_rdata_atoms(
 		db->region, db->domains, type, rdatalen, packet, &rdatas);
 	if(rdata_num == -1) {
-		log_msg(LOG_ERR, "diff: bad rdata for %s",
+		log_msg(LOG_ERR, "diff: bad rdata for %s", 
 			dname_to_string(dname,0));
 		return 0;
 	}
-	rrnum = find_rr_num(rrset, type, klass, rdatas, rdata_num);
+	rrnum = find_rr_num(rrset, type, klass, ttl, rdatas, rdata_num);
 	if(rrnum != -1) {
-		DEBUG(DEBUG_XFRD, 2, (LOG_ERR, "diff: RR <%s, %s> already exists",
-			dname_to_string(dname,0), rrtype_to_string(type)));
+		DEBUG(DEBUG_XFRD, 2, (LOG_ERR, "diff: RR %s already exists", 
+			dname_to_string(dname,0)));
 		/* ignore already existing RR: lenient accepting of messages */
 		return 1;
 	}
-
+	
 	/* re-alloc the rrs and add the new */
 	rrs_old = rrset->rrs;
-	rrset->rrs = region_alloc(db->region,
+	rrset->rrs = region_alloc(db->region, 
 		(rrset->rr_count+1) * sizeof(rr_type));
 	if(!rrset->rrs) {
 		log_msg(LOG_ERR, "out of memory, %s:%d", __FILE__, __LINE__);
@@ -565,20 +486,20 @@ add_RR(namedb_type* db, const dname_type* dname,
 			zone->updated = 1;
 			/* BUG #103 tweaked SOA ttl value */
 			if(zone->soa_nx_rrset == 0) {
-				zone->soa_nx_rrset = region_alloc(db->region,
+				zone->soa_nx_rrset = region_alloc(db->region, 
 					sizeof(rrset_type));
 				if(!zone->soa_nx_rrset) {
-					log_msg(LOG_ERR, "out of memory, %s:%d",
+					log_msg(LOG_ERR, "out of memory, %s:%d", 
 						__FILE__, __LINE__);
 					exit(1);
 				}
 				zone->soa_nx_rrset->rr_count = 1;
 				zone->soa_nx_rrset->next = 0;
 				zone->soa_nx_rrset->zone = zone;
-				zone->soa_nx_rrset->rrs = region_alloc(db->region,
+				zone->soa_nx_rrset->rrs = region_alloc(db->region, 
 					sizeof(rr_type));
 				if(!zone->soa_nx_rrset->rrs) {
-					log_msg(LOG_ERR, "out of memory, %s:%d",
+					log_msg(LOG_ERR, "out of memory, %s:%d", 
 						__FILE__, __LINE__);
 					exit(1);
 				}
@@ -589,53 +510,31 @@ add_RR(namedb_type* db, const dname_type* dname,
 			if (rrset->rrs->ttl > ntohl(soa_minimum)) {
 				rrset->zone->soa_nx_rrset->rrs[0].ttl = ntohl(soa_minimum);
 			}
-			domain->has_SOA = 1;
 		}
 		if(type == TYPE_NS) {
 			zone->ns_rrset = rrset;
 		}
+#ifdef DNSSEC
 		if(type == TYPE_RRSIG) {
 			int i;
 			for (i = 0; i < rrset->rr_count; ++i) {
-				if (rr_rrsig_type_covered(&rrset->rrs[i]) == TYPE_DNSKEY) {
+				if (rr_rrsig_type_covered(&rrset->rrs[i]) == TYPE_SOA) {
 					zone->is_secure = 1;
 					break;
 				}
 			}
 		}
+#endif
 	}
-
-#ifdef NSEC3
-#ifndef FULL_PREHASH
-	if ((type == TYPE_NSEC3) &&
-	    (rrset->rr_count == 1)) {
-		/* NSEC3 RRset just added */
-		if (0 != namedb_add_nsec3_domain(db, domain, zone))
-			return 0;
-	}
-	if (is_axfr == 0) {
-		struct domain *parent = domain;
-		do {
-			if (0 != namedb_add_nsec3_mod_domain(db, parent))
-				return 0;
-			parent = parent->parent;
-		} while (parent != zone->apex->parent);
-	}
-#else
-	(void)is_axfr;
-#endif /* !FULL_PREHASH */
-#endif /* NSEC3 */
-
 	return 1;
 }
 
-static zone_type*
+static zone_type* 
 find_zone(namedb_type* db, const dname_type* zone_name, nsd_options_t* opt,
 	size_t child_count)
 {
 	domain_type *domain;
 	zone_type* zone;
-	zone_options_t* opts;
 	domain = domain_table_find(db->domains, zone_name);
 	if(!domain) {
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "xfr: creating domain %s",
@@ -654,13 +553,6 @@ find_zone(namedb_type* db, const dname_type* zone_name, nsd_options_t* opt,
 			return zone;
 		}
 	}
-	/* lookup in config */
-	opts = zone_options_find(opt, domain_dname(domain));
-	if(!opts) {
-		log_msg(LOG_ERR, "xfr: zone %s not in config.",
-			dname_to_string(zone_name,0));
-		return 0;
-	}
 	/* create the zone */
 	DEBUG(DEBUG_XFRD,1, (LOG_INFO, "xfr: creating zone_type %s",
 		dname_to_string(zone_name,0)));
@@ -670,7 +562,6 @@ find_zone(namedb_type* db, const dname_type* zone_name, nsd_options_t* opt,
 		exit(1);
 	}
 	zone->next = db->zones;
-	zone->opts = opts;
 	db->zones = zone;
 	db->zone_count++;
 	zone->apex = domain;
@@ -687,18 +578,12 @@ find_zone(namedb_type* db, const dname_type* zone_name, nsd_options_t* opt,
 		exit(1);
 	}
 	memset(zone->dirty, 0, sizeof(uint8_t)*child_count);
-#ifdef NSEC3
-#ifndef FULL_PREHASH
-	zone->nsec3_domains = NULL;
-
-	if (0 != zone_nsec3_domains_create(db, zone)) {
-		log_msg(LOG_ERR,
-			"xfr: zone NSEC3 domains "
-			"memory allocation failure");
+	zone->opts = zone_options_find(opt, domain_dname(zone->apex)); 
+	if(!zone->opts) {
+		log_msg(LOG_ERR, "xfr: zone %s not in config.",
+			dname_to_string(zone_name,0));
 		return 0;
 	}
-#endif /* !FULL_PREHASH */
-#endif /* NSEC3 */
 	zone->number = db->zone_count;
 	zone->is_secure = 0;
 	zone->updated = 1;
@@ -706,19 +591,11 @@ find_zone(namedb_type* db, const dname_type* zone_name, nsd_options_t* opt,
 	return zone;
 }
 
-static void
+static void 
 delete_zone_rrs(namedb_type* db, zone_type* zone)
 {
 	rrset_type *rrset;
 	domain_type *domain = zone->apex;
-	domain_type *next = NULL;
-	zone->updated = 1;
-#ifdef NSEC3
-#ifndef FULL_PREHASH
-	zone_nsec3_domains_destroy(db, zone);
-#endif /* !FULL_PREHASH */
-#endif /* NSEC3 */
-
 	/* go through entire tree below the zone apex (incl subzones) */
 	while(domain && dname_is_subdomain(
 		domain_dname(domain), domain_dname(zone->apex)))
@@ -727,24 +604,12 @@ delete_zone_rrs(namedb_type* db, zone_type* zone)
 			dname_to_string(domain_dname(domain),0)));
 		/* delete all rrsets of the zone */
 		while((rrset = domain_find_any_rrset(domain, zone))) {
-			(void)rrset_delete(db, domain, rrset);
+			rrset_delete(db, domain, rrset);
 		}
-		next = domain_next(domain);
-		domain->nextdiff = next;
-		domain = next;
+		domain = domain_next(domain);
 	}
-#ifdef NSEC3
-#ifndef FULL_PREHASH
-	if (0 != zone_nsec3_domains_create(db, zone)) {
-		log_msg(LOG_ERR,
-			"Zone %s: unable to create zone NSEC3 prehash table",
-			dname_to_string(domain_dname(zone->apex),
-			NULL));
-	}
-#endif /* !FULL_PREHASH */
-#endif /* NSEC3 */
 
-	DEBUG(DEBUG_XFRD, 1, (LOG_INFO, "axfrdel: recyclebin holds %lu bytes",
+	DEBUG(DEBUG_XFRD, 1, (LOG_INFO, "axfrdel: recyclebin holds %lu bytes", 
 		(unsigned long) region_get_recycle_size(db->region)));
 #ifndef NDEBUG
 	if(nsd_debug_level >= 1)
@@ -758,28 +623,14 @@ delete_zone_rrs(namedb_type* db, zone_type* zone)
 	assert(zone->updated == 1);
 }
 
-/* fix empty terminals */
-static void
-fix_empty_terminals(zone_type* zone_db)
-{
-	domain_type* domain = zone_db->apex, *ce = NULL, *next = NULL;
-	while (domain) {
-		ce = rrset_delete_empty_terminals(domain, ce);
-		next = domain->nextdiff;
-		domain->nextdiff = NULL;
-		domain = next;
-	}
-}
-
-/* return value 0: syntaxerror,badIXFR, 1:OK, 2:done_and_skip_it */
-static int
-apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
+static int 
+apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos, 
 	const char* zone, uint32_t serialno, nsd_options_t* opt,
 	uint16_t id, uint32_t seq_nr, uint32_t seq_total,
 	int* is_axfr, int* delete_mode, int* rr_count,
 	size_t child_count)
 {
-	uint32_t filelen, msglen, pkttype, timestamp[2];
+	uint32_t filelen, msglen, pkttype;
 	int qcount, ancount, counter;
 	buffer_type* packet;
 	region_type* region;
@@ -787,14 +638,11 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 	uint16_t rrlen;
 	const dname_type *dname_zone, *dname;
 	zone_type* zone_db;
-	domain_type* last_in_list;
 	char file_zone_name[3072];
 	uint32_t file_serial, file_seq_nr;
 	uint16_t file_id;
-	off_t mempos;
 
-	memmove(&mempos, startpos, sizeof(off_t));
-	if(fseeko(in, mempos, SEEK_SET) == -1) {
+	if(fseeko(in, *startpos, SEEK_SET) == -1) {
 		log_msg(LOG_INFO, "could not fseeko: %s.", strerror(errno));
 		return 0;
 	}
@@ -804,12 +652,6 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		log_msg(LOG_ERR, "could not read type or wrong type");
 		return 0;
 	}
-	if(!diff_read_32(in, &timestamp[0]) ||
-	   !diff_read_32(in, &timestamp[1])) {
-		log_msg(LOG_ERR, "could not read timestamp");
-		return 0;
-	}
-
 	if(!diff_read_32(in, &filelen)) {
 		log_msg(LOG_ERR, "could not read len");
 		return 0;
@@ -833,14 +675,12 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		!diff_read_32(in, &file_seq_nr))
 	{
 		log_msg(LOG_ERR, "could not part data");
-		region_destroy(region);
 		return 0;
 	}
-
+	
 	if(strcmp(file_zone_name, zone) != 0 || serialno != file_serial ||
 		id != file_id || seq_nr != file_seq_nr) {
 		log_msg(LOG_ERR, "internal error: reading part with changed id");
-		region_destroy(region);
 		return 0;
 	}
 	msglen = filelen - sizeof(uint32_t)*3 - sizeof(uint16_t)
@@ -851,10 +691,9 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 	if(!zone_db) {
 		log_msg(LOG_ERR, "no zone exists");
 		region_destroy(region);
-		/* break out and stop the IXFR, ignore it */
-		return 2;
+		return 0;
 	}
-
+	
 	if(msglen > QIOBUFSZ) {
 		log_msg(LOG_ERR, "msg too long");
 		region_destroy(region);
@@ -868,7 +707,7 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 	}
 	buffer_set_limit(packet, msglen);
 
-	/* only answer section is really used, question, additional and
+	/* only answer section is really used, question, additional and 
 	   authority section RRs are skipped */
 	qcount = QDCOUNT(packet);
 	ancount = ANCOUNT(packet);
@@ -933,13 +772,11 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		*rr_count = 1;
 		*is_axfr = 0;
 		*delete_mode = 0;
-
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "diff: %s start count %d, ax %d, delmode %d",
 			dname_to_string(dname_zone, 0), *rr_count, *is_axfr, *delete_mode));
 	}
 	else  counter = 0;
 
-	last_in_list = zone_db->apex;
 	for(; counter < ancount; ++counter,++(*rr_count))
 	{
 		uint16_t type, klass;
@@ -960,7 +797,7 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		ttl = buffer_read_u32(packet);
 		rrlen = buffer_read_u16(packet);
 		if(!buffer_available(packet, rrlen)) {
-			log_msg(LOG_ERR, "bad xfr RR rdata %d, len %d have %d",
+			log_msg(LOG_ERR, "bad xfr RR rdata %d, len %d have %d", 
 				*rr_count, rrlen, (int)buffer_remaining(packet));
 			region_destroy(region);
 			return 0;
@@ -996,17 +833,6 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 				*delete_mode = 0;
 				*is_axfr = 1;
 			}
-			/* must have stuff in memory for a successful IXFR,
-			 * the serial number of the SOA has been checked
-			 * previously (by check_for_bad_serial) if it exists */
-			if(!*is_axfr && !domain_find_rrset(zone_db->apex,
-				zone_db, TYPE_SOA)) {
-				log_msg(LOG_ERR, "%s SOA serial %d is not "
-					"in memory, skip IXFR", zone, serialno);
-				region_destroy(region);
-				/* break out and stop the IXFR, ignore it */
-				return 2;
-			}
 			buffer_set_position(packet, bufpos);
 		}
 		if(type == TYPE_SOA && !*is_axfr) {
@@ -1022,41 +848,31 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 			buffer_skip(packet, rrlen);
 			continue;
 		}
-
-		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "xfr %s RR dname is %s type %s",
+		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "xfr %s RR dname is %s type %s", 
 			*delete_mode?"del":"add",
 			dname_to_string(dname,0), rrtype_to_string(type)));
 		if(*delete_mode) {
 			/* delete this rr */
 			if(!*is_axfr && type == TYPE_SOA && counter==ancount-1
-				&& seq_nr == seq_total-1) {
+				&& seq_nr == seq_total-1)
 				continue; /* do not delete final SOA RR for IXFR */
-			}
-			if(!delete_RR(db, dname, type, klass, last_in_list, packet,
-				rrlen, zone_db, region, *is_axfr)) {
-				region_destroy(region);
+			if(!delete_RR(db, dname, type, klass, ttl, packet, 
+				rrlen, zone_db, region))
 				return 0;
-			}
-			if (!*is_axfr && last_in_list->nextdiff) {
-				last_in_list = last_in_list->nextdiff;
-			}
 		}
 		else
 		{
 			/* add this rr */
-			if(!add_RR(db, dname, type, klass, ttl, packet,
-				rrlen, zone_db, *is_axfr)) {
-				region_destroy(region);
+			if(!add_RR(db, dname, type, klass, ttl, packet, 
+				rrlen, zone_db))
 				return 0;
-			}
 		}
 	}
-	fix_empty_terminals(zone_db);
 	region_destroy(region);
 	return 1;
 }
 
-static int
+static int 
 check_for_bad_serial(namedb_type* db, const char* zone_str, uint32_t old_serial)
 {
 	/* see if serial OK with in-memory serial */
@@ -1070,7 +886,7 @@ check_for_bad_serial(namedb_type* db, const char* zone_str, uint32_t old_serial)
 	if(zone && zone->apex == domain && zone->soa_rrset && old_serial)
 	{
 		uint32_t memserial;
-		memcpy(&memserial, rdata_atom_data(zone->soa_rrset->rrs[0].rdatas[2]),
+		memcpy(&memserial, rdata_atom_data(zone->soa_rrset->rrs[0].rdatas[2]), 
 			sizeof(uint32_t));
 		if(old_serial != ntohl(memserial)) {
 			region_destroy(region);
@@ -1110,7 +926,7 @@ struct diff_xfrpart {
 	off_t file_pos;
 };
 
-static struct diff_read_data*
+static struct diff_read_data* 
 diff_read_data_create()
 {
 	region_type* region = region_create(xalloc, free);
@@ -1121,7 +937,7 @@ diff_read_data_create()
 		exit(1);
 	}
 	data->region = region;
-	data->zones = rbtree_create(region,
+	data->zones = rbtree_create(region, 
 		(int (*)(const void *, const void *)) dname_compare);
 	return data;
 }
@@ -1137,9 +953,9 @@ diff_read_find_zone(struct diff_read_data* data, const char* name)
 
 static int intcompf(const void* a, const void* b)
 {
-	if(*(uint32_t*)a < *(uint32_t*)b)
+	if(*(uint32_t*)a < *(uint32_t*)b) 
 		return -1;
-	if(*(uint32_t*)a > *(uint32_t*)b)
+	if(*(uint32_t*)a > *(uint32_t*)b) 
 		return +1;
 	return 0;
 }
@@ -1200,7 +1016,7 @@ mark_and_exit(nsd_options_t* opt, FILE* f, off_t commitpos, const char* desc)
 		fclose(f);
 	} else {
 		uint8_t c = 0;
-		(void)write_data(f, &c, sizeof(c));
+		fwrite(&c, sizeof(c), 1, f);
 		fclose(f);
 		log_msg(LOG_ERR, "marked xfr as failed: %s", desc);
 		log_msg(LOG_ERR, "marked xfr so that next reload can succeed");
@@ -1208,8 +1024,8 @@ mark_and_exit(nsd_options_t* opt, FILE* f, off_t commitpos, const char* desc)
 	exit(1);
 }
 
-static int
-read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
+static int 
+read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt, 
 	struct diff_read_data* data, struct diff_log** log,
 	size_t child_count)
 {
@@ -1267,7 +1083,7 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 		return 1;
 	}
 	if(committed && check_for_bad_serial(db, zone_buf, old_serial)) {
-		DEBUG(DEBUG_XFRD,1, (LOG_ERR,
+		DEBUG(DEBUG_XFRD,1, (LOG_ERR, 
 			"skipping diff file commit with bad serial"));
 		zp->parts->root = RBTREE_NULL;
 		zp->parts->count = 0;
@@ -1282,7 +1098,7 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 		}
 	}
 	if(!have_all_parts) {
-		DEBUG(DEBUG_XFRD,1, (LOG_ERR,
+		DEBUG(DEBUG_XFRD,1, (LOG_ERR, 
 			"skipping diff file commit without all parts"));
 		if(thislog)
 			thislog->error = "error missing parts";
@@ -1293,41 +1109,7 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 		int is_axfr=0, delete_mode=0, rr_count=0;
 		off_t resume_pos;
 
-#ifdef NSEC3
-#ifndef FULL_PREHASH
-		struct region *region;
-		dname_type const *zone_dname;
-		struct zone *zone;
-
-		region = region_create(xalloc, free);
-		if (region == NULL) {
-			log_msg(LOG_ERR, "out of memory");
-			return 0;
-		}
-		zone_dname = dname_parse(region, zone_buf);
-		if (zone_dname == NULL) {
-			log_msg(LOG_ERR, "out of memory");
-		        region_destroy(region);
-			return 0;
-		}
-		zone = find_zone(db, zone_dname, opt, child_count);
-		region_destroy(region);
-		if (zone == NULL) {
-			log_msg(LOG_ERR, "no zone exists");
-			/* just stop trying applying ixfr */
-			return 1;
-		}
-		if (0 != namedb_nsec3_mod_domains_create(db)) {
-			log_msg(LOG_ERR,
-				"unable to allocate space "
-				"for modified NSEC3 domains");
-			return 0;
-		}
-#endif /* !FULL_PREHASH */
-#endif /* NSEC3 */
-
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "processing xfr: %s", log_buf));
-
 		resume_pos = ftello(in);
 		if(resume_pos == -1) {
 			log_msg(LOG_INFO, "could not ftello: %s.", strerror(errno));
@@ -1335,28 +1117,14 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 		}
 		for(i=0; i<num_parts; i++) {
 			struct diff_xfrpart *xp = diff_read_find_part(zp, i);
-			int ret;
 			DEBUG(DEBUG_XFRD,2, (LOG_INFO, "processing xfr: apply part %d", (int)i));
-			ret = apply_ixfr(db, in, &xp->file_pos, zone_buf, new_serial, opt,
+			if(!apply_ixfr(db, in, &xp->file_pos, zone_buf, new_serial, opt,
 				id, xp->seq_nr, num_parts, &is_axfr, &delete_mode,
-				&rr_count, child_count);
-			if(ret == 0) {
-				log_msg(LOG_ERR, "bad ixfr packet part %d in %s", (int)i,
-					opt->difffile);
+				&rr_count, child_count)) {
+				log_msg(LOG_ERR, "bad ixfr packet part %d", (int)i);
 				mark_and_exit(opt, in, commitpos, log_buf);
-			} else if(ret == 2) {
-				break;
 			}
 		}
-#ifdef NSEC3
-#ifndef FULL_PREHASH
-		if (is_axfr != 0)
-			prehash_zone(db, zone);
-		else
-			prehash_zone_incremental(db, zone);
-#endif /* !FULL_PREHASH */
-#endif /* NSEC3 */
-
 		if(fseeko(in, resume_pos, SEEK_SET) == -1) {
 			log_msg(LOG_INFO, "could not fseeko: %s.", strerror(errno));
 			return 0;
@@ -1365,7 +1133,7 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 	else {
 	 	DEBUG(DEBUG_XFRD,1, (LOG_INFO, "skipping xfr: %s", log_buf));
 	}
-
+	
 	/* clean out the parts for the zone after the commit/rollback */
 	zp->parts->root = RBTREE_NULL;
 	zp->parts->count = 0;
@@ -1384,7 +1152,7 @@ store_ixfr_data(FILE *in, uint32_t len, struct diff_read_data* data, off_t* star
 		!diff_read_32(in, &new_serial) ||
 		!diff_read_16(in, &id) ||
 		!diff_read_32(in, &seq)) {
-		log_msg(LOG_INFO, "could not read ixfr store info: file format error");
+		log_msg(LOG_INFO, "could not read ixfr store info: %s", strerror(errno));
 		return 0;
 	}
 	len -= sizeof(uint32_t)*3 + sizeof(uint16_t) + strlen(zone_name);
@@ -1408,17 +1176,15 @@ store_ixfr_data(FILE *in, uint32_t len, struct diff_read_data* data, off_t* star
 	return 1;
 }
 
-static int
+static int 
 read_process_part(namedb_type* db, FILE *in, uint32_t type,
 	nsd_options_t* opt, struct diff_read_data* data,
 	struct diff_log** log, size_t child_count, off_t* startpos)
 {
 	uint32_t len, len2;
 
-	/* read length */
-	if(!diff_read_32(in, &len))
-		return 1;
-	/* read content */
+	if(!diff_read_32(in, &len)) return 1;
+
 	if(type == DIFF_PART_IXFR) {
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "part IXFR len %d", len));
 		if(!store_ixfr_data(in, len, data, startpos))
@@ -1426,16 +1192,14 @@ read_process_part(namedb_type* db, FILE *in, uint32_t type,
 	}
 	else if(type == DIFF_PART_SURE) {
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "part SURE len %d", len));
-		if(!read_sure_part(db, in, opt, data, log, child_count))
+		if(!read_sure_part(db, in, opt, data, log, child_count)) 
 			return 0;
 	} else {
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "unknown part %x len %d", type, len));
 		return 0;
 	}
-	/* read length */
 	if(!diff_read_32(in, &len2))
 		return 1; /* short read is OK */
-	/* verify length */
 	if(len != len2)
 		return 0; /* bad data is wrong */
 	return 1;
@@ -1451,8 +1215,6 @@ find_smallest_offset(struct diff_read_data* data, off_t* offset)
 	int found_any = 0;
 	struct diff_zone* dz;
 	struct diff_xfrpart* dx;
-	off_t mem_offset, mem_fpos;
-
 	if(!data || !data->zones)
 		return 0;
 	RBTREE_FOR(dz, struct diff_zone*, data->zones)
@@ -1461,16 +1223,12 @@ find_smallest_offset(struct diff_read_data* data, off_t* offset)
 			continue;
 		RBTREE_FOR(dx, struct diff_xfrpart*, dz->parts)
 		{
-			memmove(&mem_fpos, &dx->file_pos, sizeof(off_t));
-
 			if(found_any) {
-				memmove(&mem_offset, offset, sizeof(off_t));
-
-				if(mem_fpos < mem_offset)
-					memmove(offset, &mem_fpos, sizeof(off_t));
+				if(dx->file_pos < *offset)
+					memmove(offset, &dx->file_pos, sizeof(off_t));
 			} else {
 				found_any = 1;
-				memmove(offset, &mem_fpos, sizeof(off_t));
+				memmove(offset, &dx->file_pos, sizeof(off_t));
 			}
 		}
 	}
@@ -1478,13 +1236,13 @@ find_smallest_offset(struct diff_read_data* data, off_t* offset)
 	return found_any;
 }
 
-int
+int 
 diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 	size_t child_count)
 {
 	const char* filename = opt->difffile;
 	FILE *df;
-	uint32_t type, timestamp[2], curr_timestamp[2];
+	uint32_t type;
 	struct diff_read_data* data = diff_read_data_create();
 	off_t startpos;
 
@@ -1495,96 +1253,33 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 		region_destroy(data->region);
 		return 1;
 	}
-
-	/* check timestamp */
-	curr_timestamp[0] = (uint32_t) db->diff_timestamp.tv_sec;
-	curr_timestamp[1] = (uint32_t) db->diff_timestamp.tv_usec;
-
-	if(!diff_read_32(df, &type)) {
-		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "difffile %s is empty",
-			filename));
-		db->diff_skip = 0;
-		db->diff_pos = 0;
-	}
-	else if (!diff_read_32(df, &timestamp[0]) ||
-		 !diff_read_32(df, &timestamp[1])) {
-		log_msg(LOG_ERR, "difffile %s bad first part: no timestamp",
-			filename);
-		region_destroy(data->region);
-		fclose(df);
-		return 0;
-	}
-	else if (curr_timestamp[0] != timestamp[0] ||
-		 curr_timestamp[1] != timestamp[1]) {
-		/* new timestamp, no skipping */
-		db->diff_timestamp.tv_sec = (time_t) timestamp[0];
-		db->diff_timestamp.tv_usec = (suseconds_t) timestamp[1];
-
-		if (db->diff_skip) {
-			DEBUG(DEBUG_XFRD,1, (LOG_INFO, "new timestamp on "
-				"difffile %s, restoring diff_skip and diff_pos "
-				"[old timestamp: %u.%u; new timestamp: %u.%u]",
-				filename, curr_timestamp[0], curr_timestamp[1],
-				timestamp[0], timestamp[1]));
-			db->diff_skip = 0;
-			db->diff_pos = 0;
-		}
-	}
-
-	/* Always seek, to diff_pos or to beginning of the file. */
-	if (fseeko(df, 0, SEEK_SET)==-1) {
-		log_msg(LOG_INFO, "could not fseeko file %s: %s.", filename,
-				strerror(errno));
-		region_destroy(data->region);
-		fclose(df);
-		return 0;
-	}
 	if(db->diff_skip) {
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "skip diff file"));
 		if(fseeko(df, db->diff_pos, SEEK_SET)==-1) {
-			log_msg(LOG_INFO, "could not fseeko file %s: %s. "
-					  "Reread from start.", filename,
-				strerror(errno));
+			log_msg(LOG_INFO, "could not fseeko file %s: %s. Reread from start.",
+				filename, strerror(errno));
 		}
 	}
 
 	startpos = ftello(df);
 	if(startpos == -1) {
 		log_msg(LOG_INFO, "could not ftello: %s.", strerror(errno));
-		region_destroy(data->region);
-		fclose(df);
 		return 0;
 	}
 
-	DEBUG(DEBUG_XFRD,1, (LOG_INFO, "start of diff file read at pos %u",
-		(uint32_t) db->diff_pos));
-	while(diff_read_32(df, &type))
+	while(diff_read_32(df, &type)) 
 	{
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "iter loop"));
-
-		/* read timestamp */
-		if(!diff_read_32(df, &timestamp[0]) ||
-			!diff_read_32(df, &timestamp[1])) {
-			log_msg(LOG_INFO, "could not read timestamp: %s.",
-				strerror(errno));
-			region_destroy(data->region);
-			fclose(df);
-			return 0;
-		}
-
-		if(!read_process_part(db, df, type, opt, data, log,
+		if(!read_process_part(db, df, type, opt, data, log, 
 			child_count, &startpos))
 		{
 			log_msg(LOG_INFO, "error processing diff file");
 			region_destroy(data->region);
-			fclose(df);
 			return 0;
 		}
 		startpos = ftello(df);
 		if(startpos == -1) {
 			log_msg(LOG_INFO, "could not ftello: %s.", strerror(errno));
-			region_destroy(data->region);
-			fclose(df);
 			return 0;
 		}
 	}
@@ -1592,11 +1287,9 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 
 	if(find_smallest_offset(data, &db->diff_pos)) {
 		/* can skip to the first unused element */
-		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "next time skip diff file"));
 		db->diff_skip = 1;
 	} else {
 		/* all processed, can skip to here next time */
-		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "next time skip diff file"));
 		db->diff_skip = 1;
 		db->diff_pos = ftello(df);
 		if(db->diff_pos == -1) {
@@ -1605,7 +1298,7 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 			db->diff_skip = 0;
 		}
 	}
-
+	
 	region_destroy(data->region);
 	fclose(df);
 	return 1;
@@ -1659,7 +1352,7 @@ void diff_snip_garbage(namedb_type* db, nsd_options_t* opt)
 	if(db->diff_skip) {
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "garbage collect skip diff file"));
 		if(fseeko(df, db->diff_pos, SEEK_SET)==-1) {
-			log_msg(LOG_INFO, "could not fseeko file %s: %s.",
+			log_msg(LOG_INFO, "could not fseeko file %s: %s.", 
 				filename, strerror(errno));
 			fclose(df);
 			return;
@@ -1670,12 +1363,13 @@ void diff_snip_garbage(namedb_type* db, nsd_options_t* opt)
 	if(diff_broken(df, &break_pos))
 	{
 		/* snip off at break_pos */
-		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "snipping off trailing partial part of %s",
+		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "snipping off trailing partial part of %s", 
 			filename));
 		if(ftruncate(fileno(df), break_pos) == -1)
-			log_msg(LOG_ERR, "ftruncate %s failed: %s",
+			log_msg(LOG_ERR, "ftruncate %s failed: %s", 
 				filename, strerror(errno));
 	}
 
 	fclose(df);
 }
+
