@@ -1,22 +1,20 @@
 /*
  * checkconf - Read and repeat configuration file to output.
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
  */
-#include "config.h"
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <limits.h>
-#include "tsig.h"
 #include "options.h"
 #include "util.h"
 #include "dname.h"
-#include "rrl.h"
 
 extern char *optarg;
 extern int optind;
@@ -27,14 +25,6 @@ extern int optind;
 		return; 			\
 	}
 
-#define ZONE_GET_OUTGOING(NAME, VAR)			\
-	if (strcasecmp(#NAME, (VAR)) == 0) {		\
-		acl_options_t* acl; 			\
-		for(acl=zone->NAME; acl; acl=acl->next)	\
-			quote(acl->ip_address_spec);	\
-		return; 				\
-	}
-
 #define ZONE_GET_STR(NAME, VAR) 		\
 	if (strcasecmp(#NAME, (VAR)) == 0) { 	\
 		quote(zone->NAME); 		\
@@ -46,12 +36,6 @@ extern int optind;
 		printf("%s\n", zone->NAME?"yes":"no"); 	\
 	}
 
-#define ZONE_GET_RRL(NAME, VAR, PATTERN) 			\
-	if (strcasecmp(#NAME, (VAR)) == 0) { 		\
-		zone_print_rrl_whitelist("", PATTERN->NAME);	\
-		return;					\
-	}
-
 #define SERV_GET_BIN(NAME, VAR) 			\
 	if (strcasecmp(#NAME, (VAR)) == 0) { 		\
 		printf("%s\n", opt->NAME?"yes":"no"); 	\
@@ -65,7 +49,7 @@ extern int optind;
 
 #define SERV_GET_INT(NAME, VAR) 		\
 	if (strcasecmp(#NAME, (VAR)) == 0) { 	\
-		printf("%d\n", (int) opt->NAME); 	\
+		printf("%d\n", opt->NAME); 	\
 		return; 			\
 	}
 
@@ -78,21 +62,6 @@ extern int optind;
 		return;						\
 	}
 
-#ifdef RATELIMIT
-static void zone_print_rrl_whitelist(const char* s, uint16_t w)
-{
-	int i;
-	if(w==rrl_type_all) {
-		printf("%sall\n", s);
-		return;
-	}
-	for(i=0x01; i <= 0x80; i<<=1) {
-		if( (w&i) )
-			printf("%s%s\n", s, rrltype2str(i));
-	}
-}
-#endif /* RATELIMIT */
-
 static char buf[BUFSIZ];
 
 static char *
@@ -107,7 +76,7 @@ underscore(const char *s) {
 			buf[i++] = *j;
 		}
 		j++;
-		if (i >= BUFSIZ) {
+		if (i > BUFSIZ) {
 			return NULL;
 		}
 	}
@@ -121,7 +90,7 @@ usage(void)
 	fprintf(stderr, "usage: nsd-checkconf [-v|-h] [-o option] [-z zonename]\n");
 	fprintf(stderr, "                     [-s keyname] <configfilename>\n");
 	fprintf(stderr, "       Checks NSD configuration file for errors.\n");
-	fprintf(stderr, "       Version %s. Report bugs to <%s>.\n\n",
+	fprintf(stderr, "       Version %s. Report bugs to <%s>.\n\n", 
 		PACKAGE_VERSION, PACKAGE_BUGREPORT);
 	fprintf(stderr, "Use with a configfile as argument to check syntax.\n");
 	fprintf(stderr, "Use with -o, -z or -s options to query the configuration.\n\n");
@@ -129,17 +98,16 @@ usage(void)
 	fprintf(stderr, "-h		Print this help information.\n");
 	fprintf(stderr, "-o option	Print value of the option specified to stdout.\n");
 	fprintf(stderr, "-z zonename	Print option value for the zone given.\n");
-	fprintf(stderr, "-a keyname	Print algorithm name for the TSIG key.\n");
 	fprintf(stderr, "-s keyname	Print base64 secret blob for the TSIG key.\n");
 	exit(1);
 }
 
-static void
+static void 
 print_string_var(const char* varname, const char* value)
 {
 	if (!value) {
 		printf("\t#%s\n", varname);
-	} else {
+	} else { 
 		printf("\t%s \"%s\"\n", varname, value);
 	}
 }
@@ -153,7 +121,7 @@ quote(const char *v)
 		printf("%s\n", v);
 }
 
-static void
+static void 
 quote_acl(acl_options_t* acl)
 {
 	while(acl)
@@ -165,7 +133,7 @@ quote_acl(acl_options_t* acl)
 	}
 }
 
-static void
+static void 
 print_acl(const char* varname, acl_options_t* acl)
 {
 	while(acl)
@@ -173,8 +141,6 @@ print_acl(const char* varname, acl_options_t* acl)
 		printf("\t%s ", varname);
 		if(acl->use_axfr_only)
 			printf("AXFR ");
-		if(acl->allow_udp)
-			printf("UDP ");
 		printf("%s %s\n", acl->ip_address_spec,
 			acl->nokey?"NOKEY":(acl->blocked?"BLOCKED":
 			(acl->key_name?acl->key_name:"(null)")));
@@ -213,18 +179,9 @@ print_acl(const char* varname, acl_options_t* acl)
 	}
 }
 
-static void
-print_acl_ips(const char* varname, acl_options_t* acl)
-{
-	while(acl)
-	{
-		printf("\t%s %s\n", varname, acl->ip_address_spec);
-		acl=acl->next;
-	}
-}
 
 void
-config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o, const char *z)
+config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *z)
 {
 	zone_options_t* zone;
 	ip_address_option_t* ip;
@@ -234,11 +191,7 @@ config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o, const
 		key_options_t* key = opt->keys;
 		for( ; key ; key=key->next) {
 			if(strcmp(key->name, k) == 0) {
-				if (s) {
-					quote(key->secret);
-				} else {
-					quote(key->algorithm);
-				}
+				quote(key->secret);
 				return;
 			}
 		}
@@ -267,12 +220,6 @@ config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o, const
 				ZONE_GET_ACL(provide_xfr, o);
 				ZONE_GET_ACL(allow_notify, o);
 				ZONE_GET_ACL(notify, o);
-				ZONE_GET_BIN(notify_retry, o);
-				ZONE_GET_OUTGOING(outgoing_interface, o);
-				ZONE_GET_BIN(allow_axfr_fallback, o);
-#ifdef RATELIMIT
-				ZONE_GET_RRL(rrl_whitelist, o, zone);
-#endif
 				printf("Zone option not handled: %s %s\n", z, o);
 				exit(1);
 			}
@@ -286,11 +233,9 @@ config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o, const
 		SERV_GET_BIN(debug_mode, o);
 		SERV_GET_BIN(ip4_only, o);
 		SERV_GET_BIN(ip6_only, o);
-		SERV_GET_BIN(hide_version, o);
 		/* str */
 		SERV_GET_STR(database, o);
 		SERV_GET_STR(identity, o);
-		SERV_GET_STR(nsid, o);
 		SERV_GET_STR(logfile, o);
 		SERV_GET_STR(pidfile, o);
 		SERV_GET_STR(chroot, o);
@@ -299,24 +244,11 @@ config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o, const
 		SERV_GET_STR(difffile, o);
 		SERV_GET_STR(xfrdfile, o);
 		SERV_GET_STR(port, o);
-#if defined(BIND8_STATS) && defined(USE_ZONE_STATS)
-		SERV_GET_STR(zonestatsfile, o);
-#endif
 		/* int */
 		SERV_GET_INT(server_count, o);
 		SERV_GET_INT(tcp_count, o);
-		SERV_GET_INT(tcp_query_count, o);
-		SERV_GET_INT(tcp_timeout, o);
-		SERV_GET_INT(ipv4_edns_size, o);
-		SERV_GET_INT(ipv6_edns_size, o);
 		SERV_GET_INT(statistics, o);
 		SERV_GET_INT(xfrd_reload_timeout, o);
-		SERV_GET_INT(verbosity, o);
-#ifdef RATELIMIT
-		SERV_GET_INT(rrl_size, o);
-		SERV_GET_INT(rrl_ratelimit, o);
-		SERV_GET_INT(rrl_whitelist_ratelimit, o);
-#endif
 
 		if(strcasecmp(o, "zones") == 0) {
 			RBTREE_FOR(zone, zone_options_t*, opt->zone_options)
@@ -328,7 +260,7 @@ config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o, const
 	}
 }
 
-void
+void 
 config_test_print_server(nsd_options_t* opt)
 {
 	ip_address_option_t* ip;
@@ -340,23 +272,14 @@ config_test_print_server(nsd_options_t* opt)
 	printf("\tdebug-mode: %s\n", opt->debug_mode?"yes":"no");
 	printf("\tip4-only: %s\n", opt->ip4_only?"yes":"no");
 	printf("\tip6-only: %s\n", opt->ip6_only?"yes":"no");
-	printf("\thide-version: %s\n", opt->hide_version?"yes":"no");
 	print_string_var("database:", opt->database);
 	print_string_var("identity:", opt->identity);
-	print_string_var("nsid:", opt->nsid);
 	print_string_var("logfile:", opt->logfile);
 	printf("\tserver_count: %d\n", opt->server_count);
 	printf("\ttcp_count: %d\n", opt->tcp_count);
-	printf("\ttcp_query_count: %d\n", opt->tcp_query_count);
-	printf("\ttcp_timeout: %d\n", opt->tcp_timeout);
-	printf("\tipv4-edns-size: %d\n", (int) opt->ipv4_edns_size);
-	printf("\tipv6-edns-size: %d\n", (int) opt->ipv6_edns_size);
 	print_string_var("pidfile:", opt->pidfile);
 	print_string_var("port:", opt->port);
 	printf("\tstatistics: %d\n", opt->statistics);
-#if defined(BIND8_STATS) && defined(USE_ZONE_STATS)
-	printf("\tzone-stats-file: %s\n", opt->zonestatsfile);
-#endif
 	print_string_var("chroot:", opt->chroot);
 	print_string_var("username:", opt->username);
 	print_string_var("zonesdir:", opt->zonesdir);
@@ -364,11 +287,6 @@ config_test_print_server(nsd_options_t* opt)
 	print_string_var("xfrdfile:", opt->xfrdfile);
 	printf("\txfrd_reload_timeout: %d\n", opt->xfrd_reload_timeout);
 	printf("\tverbosity: %d\n", opt->verbosity);
-#ifdef RATELIMIT
-	printf("\trrl-size: %d\n", (int)opt->rrl_size);
-	printf("\trrl-ratelimit: %d\n", (int)opt->rrl_ratelimit);
-	printf("\trrl-whitelist-ratelimit: %d\n", (int)opt->rrl_whitelist_ratelimit);
-#endif
 
 	for(ip = opt->ip_addresses; ip; ip=ip->next)
 	{
@@ -386,21 +304,15 @@ config_test_print_server(nsd_options_t* opt)
 		printf("\nzone:\n");
 		print_string_var("name:", zone->name);
 		print_string_var("zonefile:", zone->zonefile);
-#ifdef RATELIMIT
-		zone_print_rrl_whitelist("\trrl-whitelist: ", zone->rrl_whitelist);
-#endif
 		print_acl("allow-notify:", zone->allow_notify);
 		print_acl("request-xfr:", zone->request_xfr);
-		printf("\tnotify-retry: %d\n", zone->notify_retry);
 		print_acl("notify:", zone->notify);
 		print_acl("provide-xfr:", zone->provide_xfr);
-		print_acl_ips("outgoing-interface:", zone->outgoing_interface);
-		printf("\tallow-axfr-fallback: %s\n", zone->allow_axfr_fallback?"yes":"no");
 	}
-
+	
 }
 
-static int
+static int 
 additional_checks(nsd_options_t* opt, const char* filename)
 {
 	ip_address_option_t* ip = opt->ip_addresses;
@@ -412,7 +324,7 @@ additional_checks(nsd_options_t* opt, const char* filename)
 		num++;
 		ip = ip->next;
 	}
-	if(num > MAX_INTERFACES) {
+	if(num >= MAX_INTERFACES) {
 		fprintf(stderr, "%s: too many interfaces (ip-address:) specified.\n", filename);
 		errors ++;
 	}
@@ -447,10 +359,9 @@ additional_checks(nsd_options_t* opt, const char* filename)
 			fprintf(stderr, "%s: cannot base64 decode tsig secret: for key %s.\n", filename, key->name);
 			errors ++;
 		}
-		if(tsig_get_algorithm_by_name(key->algorithm) == NULL)
+		if(strcmp(key->algorithm, "hmac-md5") != 0)
 		{
-			fprintf(stderr, "%s: bad tsig algorithm %s: for key \
-%s.\n", filename, key->algorithm, key->name);
+			fprintf(stderr, "%s: bad tsig algorithm: for key %s.\n", filename, key->name);
 			errors ++;
 		}
 	}
@@ -458,24 +369,15 @@ additional_checks(nsd_options_t* opt, const char* filename)
 #ifndef BIND8_STATS
 	if(opt->statistics > 0)
 	{
-		fprintf(stderr, "%s: 'statistics: %d' but BIND 8 statistics feature not enabled.\n",
+		fprintf(stderr, "%s: 'statistics: %d' but BIND 8 statistics feature not enabled.\n", 
 			filename, opt->statistics);
 		errors ++;
 	}
-#  ifndef USE_ZONE_STATS
-	if(opt->zonestatsfile)
-	{
-		fprintf(stderr, "%s: 'zone-stats-file: %s' but per zone BIND 8 statistics feature not enabled.\n",
-			filename, opt->zonestatsfile);
-		errors ++;
-	}
-#  endif
 #endif
-
 #ifndef HAVE_CHROOT
 	if(opt->chroot != 0)
 	{
-		fprintf(stderr, "%s: chroot %s given. chroot not supported on this platform.\n",
+		fprintf(stderr, "%s: chroot %s given. chroot not supported on this platform.\n", 
 			filename, opt->chroot);
 		errors ++;
 	}
@@ -492,57 +394,53 @@ additional_checks(nsd_options_t* opt, const char* filename)
                 int l = strlen(opt->chroot);
 
                 if (strncmp(opt->chroot, opt->pidfile, l) != 0) {
-			fprintf(stderr, "%s: pidfile %s is not relative to chroot %s.\n",
+			fprintf(stderr, "%s: pidfile %s is not relative to chroot %s.\n", 
 				filename, opt->pidfile, opt->chroot);
 			errors ++;
-                }
+                } 
 		if (strncmp(opt->chroot, opt->database, l) != 0) {
-			fprintf(stderr, "%s: databasefile %s is not relative to chroot %s.\n",
+			fprintf(stderr, "%s: databasefile %s is not relative to chroot %s.\n", 
 				filename, opt->database, opt->chroot);
 			errors ++;
                 }
 		if (strncmp(opt->chroot, opt->difffile, l) != 0) {
-			fprintf(stderr, "%s: difffile %s is not relative to chroot %s.\n",
+			fprintf(stderr, "%s: difffile %s is not relative to chroot %s.\n", 
 				filename, opt->difffile, opt->chroot);
 			errors ++;
                 }
 		if (strncmp(opt->chroot, opt->xfrdfile, l) != 0) {
-			fprintf(stderr, "%s: xfrdfile %s is not relative to chroot %s.\n",
+			fprintf(stderr, "%s: xfrdfile %s is not relative to chroot %s.\n", 
 				filename, opt->xfrdfile, opt->chroot);
 			errors ++;
                 }
         }
 	if (atoi(opt->port) <= 0) {
-		fprintf(stderr, "%s: port number '%s' is not a positive number.\n",
+		fprintf(stderr, "%s: port number '%s' is not a positive number.\n", 
 			filename, opt->port);
 		errors ++;
 	}
 	if(errors != 0) {
 		fprintf(stderr, "%s: %d semantic errors in %d zones, %d keys.\n",
-			filename, errors, (int)nsd_options_num_zones(opt),
+			filename, errors, (int)nsd_options_num_zones(opt), 
 			(int)opt->numkeys);
 	}
-
+	
 	return (errors == 0);
 }
 
-int
+int 
 main(int argc, char* argv[])
 {
 	int c;
-	int verbose = 0;
-	int key_sec = 0;
+	int verbose = 0; 
 	const char * conf_opt = NULL; /* what option do you want? Can be NULL -> print all */
 	const char * conf_zone = NULL; /* what zone are we talking about */
 	const char * conf_key = NULL; /* what key is needed */
 	const char* configfile;
 	nsd_options_t *options;
 
-	log_init("nsd-checkconf");
-
-
         /* Parse the command line... */
-        while ((c = getopt(argc, argv, "vo:a:s:z:")) != -1) {
+        while ((c = getopt(argc, argv, "vo:s:z:")) != -1) {
 		switch (c) {
 		case 'v':
 			verbose = 1;
@@ -550,20 +448,8 @@ main(int argc, char* argv[])
 		case 'o':
 			conf_opt = optarg;
 			break;
-		case 'a':
-			if (conf_key) {
-				fprintf(stderr, "Error: cannot combine -a with -s or other -a.\n");
-				exit(1);
-			}
-			conf_key = optarg;
-			break;
 		case 's':
-			if (conf_key) {
-				fprintf(stderr, "Error: cannot combine -s with -a or other -s.\n");
-				exit(1);
-			}
 			conf_key = optarg;
-			key_sec = 1;
 			break;
 		case 'z':
 			conf_zone = optarg;
@@ -581,18 +467,17 @@ main(int argc, char* argv[])
 
 	/* read config file */
 	options = nsd_options_create(region_create(xalloc, free));
-	tsig_init(options->region);
 	if (!parse_options_file(options, configfile) ||
 	   !additional_checks(options, configfile)) {
 		exit(2);
 	}
 	if (conf_opt || conf_key) {
-		config_print_zone(options, conf_key, key_sec, underscore(conf_opt), conf_zone);
+		config_print_zone(options, conf_key, underscore(conf_opt), conf_zone);
 	} else {
 		if (verbose) {
-			printf("# Read file %s: %d zones, %d keys.\n",
-				configfile,
-				(int)nsd_options_num_zones(options),
+			printf("# Read file %s: %d zones, %d keys.\n", 
+				configfile, 
+				(int)nsd_options_num_zones(options), 
 				(int)options->numkeys);
 			config_test_print_server(options);
 		}
