@@ -1,7 +1,7 @@
 /*
  * nsd-xfer.c -- nsd-xfer(8).
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -14,7 +14,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include <limits.h>
 #include <assert.h>
 #include <errno.h>
 #include <netdb.h>
@@ -28,10 +27,12 @@
 #include "dns.h"
 #include "packet.h"
 #include "query.h"
+#include "rdata.h"
 #include "region-allocator.h"
 #include "tsig.h"
 #include "tsig-openssl.h"
 #include "util.h"
+#include "zonec.h"
 
 
 /*
@@ -44,9 +45,12 @@
  * Exit codes are based on named-xfer for now.  See ns_defs.h in
  * bind8.
  */
-#define XFER_UPTODATE	0
-#define XFER_SUCCESS	1
-#define XFER_FAIL		3
+enum nsd_xfer_exit_codes
+{
+	XFER_UPTODATE = 0,
+	XFER_SUCCESS  = 1,
+	XFER_FAIL     = 3
+};
 
 struct axfr_state
 {
@@ -105,12 +109,6 @@ int check_matching_address_family(struct addrinfo *a, struct addrinfo *b);
  */
 struct addrinfo *find_by_address_family(struct addrinfo *addrs, int family);
 
-/*
- * Assigns pointers to hostname and port and wipes out the optional delimiter.
- */
-void get_hostname_port_frm_str(char* arg, const char** hostname,
-	const char** port);
-
 /*
  * Log an error message and exit.
  */
@@ -149,14 +147,13 @@ usage (void)
 	fprintf(stderr,
 		"Usage: nsd-xfer [OPTION]... -z zone -f file server...\n"
 		"NSD AXFR client.\n\nSupported options:\n"
-		"  -4            Only use IPv4 connections.\n"
-		"  -6            Only use IPv6 connections.\n"
-		"  -a src[@port] Local hostname/ip-address for the \
-connection, including optional source port.\n"
-		"  -f file       Output zone file name.\n"
-		"  -p port       The port to connect to.\n"
-		"  -s serial     The current zone serial.\n"
-		"  -T tsiginfo   The TSIG key file name.  The file is removed "
+		"  -4           Only use IPv4 connections.\n"
+		"  -6           Only use IPv6 connections.\n"
+		"  -a src       Local hostname/IP for the connection.\n"
+		"  -f file      Output zone file name.\n"
+		"  -p port      The port to connect to.\n"
+		"  -s serial    The current zone serial.\n"
+		"  -T tsiginfo  The TSIG key file name.  The file is removed "
 		"after reading the\n               key.\n"
 		"  -v           Verbose output.\n");
 	fprintf(stderr,
@@ -187,7 +184,7 @@ to_alarm(int ATTR_UNUSED(sig))
  * leading and trailing whitespace and non-zero is returned.
  */
 static int
-tsig_read_line(FILE *in, char *line, size_t size)
+read_line(FILE *in, char *line, size_t size)
 {
 	if (!fgets(line, size, in)) {
 		return 0;
@@ -198,26 +195,23 @@ tsig_read_line(FILE *in, char *line, size_t size)
 }
 
 static tsig_key_type *
-read_tsig_key_data(region_type *region, FILE *in,
-	int ATTR_UNUSED(default_family), tsig_algorithm_type** tsig_algo)
+read_tsig_key_data(region_type *region, FILE *in, 
+	int ATTR_UNUSED(default_family))
 {
 	char line[4000];
 	tsig_key_type *key = (tsig_key_type *) region_alloc(
 		region, sizeof(tsig_key_type));
 	int size;
-	uint8_t algo = 0;
 	uint8_t data[4000];
 
-	/* server address */
-	if (!tsig_read_line(in, line, sizeof(line))) {
+	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key server address: '%s'",
 		      strerror(errno));
 		return NULL;
 	}
-	/* server address unused */
+	/* server name unused */
 
-	/* tsig keyname */
-	if (!tsig_read_line(in, line, sizeof(line))) {
+	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key name: '%s'", strerror(errno));
 		return NULL;
 	}
@@ -228,20 +222,12 @@ read_tsig_key_data(region_type *region, FILE *in,
 		return NULL;
 	}
 
-	/* tsig algorithm */
-	if (!tsig_read_line(in, line, sizeof(line))) {
-		error("failed to read TSIG key algorithm: '%s'", strerror(errno));
-		return NULL;
-	}
-	algo = (uint8_t) atoi((const char*) line);
-	*tsig_algo = tsig_get_algorithm_by_id(algo);
-	if (*tsig_algo == NULL) {
-		error("failed to parse TSIG key algorithm %i: '%s'\n", algo, strerror(errno));
+	if (!read_line(in, line, sizeof(line))) {
+		error("failed to read TSIG key type: '%s'", strerror(errno));
 		return NULL;
 	}
 
-	/* tsig secret */
-	if (!tsig_read_line(in, line, sizeof(line))) {
+	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key data: '%s'\n", strerror(errno));
 		return NULL;
 	}
@@ -264,7 +250,7 @@ read_tsig_key_data(region_type *region, FILE *in,
 static tsig_key_type *
 read_tsig_key(region_type *region,
 	      const char *tsiginfo_filename,
-	      int default_family, tsig_algorithm_type** algo)
+	      int default_family)
 {
 	FILE *in;
 	tsig_key_type *key;
@@ -277,7 +263,7 @@ read_tsig_key(region_type *region,
 		return NULL;
 	}
 
-	key = read_tsig_key_data(region, in, default_family, algo);
+	key = read_tsig_key_data(region, in, default_family);
 
 	fclose(in);
 
@@ -698,7 +684,7 @@ init_query(query_type *q,
 	   uint16_t klass,
 	   tsig_record_type *tsig)
 {
-	uint16_t query_id = qid_generate();
+	uint16_t query_id = (uint16_t) random();
 
 	buffer_clear(q->packet);
 
@@ -775,7 +761,6 @@ main(int argc, char *argv[])
 	const char *local_hostname = NULL;
 	struct addrinfo *local_address, *local_addresses = NULL;
 	const char *port = TCP_PORT;
-	const char *local_port = NULL;
 	int default_family = DEFAULT_AI_FAMILY;
 	struct sigaction mysigaction;
 	FILE *zone_file;
@@ -827,8 +812,7 @@ main(int argc, char *argv[])
 #endif /* !INET6 */
 			break;
 		case 'a':
-			get_hostname_port_frm_str((char *) optarg,
-				&local_hostname, &local_port);
+			local_hostname = optarg;
 			break;
 		case 'f':
 			zone_filename = optarg;
@@ -840,16 +824,13 @@ main(int argc, char *argv[])
 			port = optarg;
 			break;
 		case 's': {
-			uint32_t v;
 			const char *t;
- 			state.first_transfer = 0;
-			v = strtoserial(optarg, &t);
-			if (optarg[0] == '\0' || *t != '\0')
-			{
+			state.first_transfer = 0;
+			state.last_serial = strtottl(optarg, &t);
+			if (*t != '\0') {
 				error("bad serial '%s'", optarg);
 				exit(XFER_FAIL);
 			}
-			state.last_serial = v;
 			break;
 		}
 		case 'T':
@@ -876,11 +857,17 @@ main(int argc, char *argv[])
 		usage();
 
 	if (tsig_key_filename) {
-		tsig_algorithm_type *tsig_algo = NULL;
+		tsig_algorithm_type *md5
+			= tsig_get_algorithm_by_name("hmac-md5");
+		if (!md5) {
+			error("cannot initialize hmac-md5: TSIG support not"
+			      " enabled");
+			exit(XFER_FAIL);
+		}
+
 		tsig_key = read_tsig_key(
-			region, tsig_key_filename, default_family, &tsig_algo);
+			region, tsig_key_filename, default_family);
 		if (!tsig_key) {
-			error("cannot initialize TSIG: error in tsiginfo file");
 			exit(XFER_FAIL);
 		}
 
@@ -889,7 +876,7 @@ main(int argc, char *argv[])
 		state.tsig = (tsig_record_type *) region_alloc(
 			region, sizeof(tsig_record_type));
 		tsig_create_record(state.tsig, region);
-		tsig_init_record(state.tsig, tsig_algo, tsig_key);
+		tsig_init_record(state.tsig, md5, tsig_key);
 	}
 
 	mysigaction.sa_handler = to_alarm;
@@ -905,21 +892,23 @@ main(int argc, char *argv[])
 	hints.ai_protocol = IPPROTO_TCP;
 
 	if (local_hostname) {
-		int rc = getaddrinfo(local_hostname, local_port,
+		int rc = getaddrinfo(local_hostname, NULL,
 				     &hints, &local_addresses);
 		if (rc) {
 			error("local hostname '%s' not found: %s",
-				local_hostname, gai_strerror(rc));
+			      local_hostname,
+			      gai_strerror(rc));
 		}
 	}
 
-	for (/*empty*/; *argv; ++argv) {
+	for (; *argv; ++argv) {
 		/* Try each server separately until one succeeds.  */
 		int rc;
 
 		rc = getaddrinfo(*argv, port, &hints, &res0);
 		if (rc) {
-			warning("skipping bad address %s: %s\n", *argv,
+			warning("skipping bad address %s: %s\n",
+				*argv,
 				gai_strerror(rc));
 			continue;
 		}
@@ -934,7 +923,7 @@ main(int argc, char *argv[])
 		}
 
 		for (res = res0; res; res = res->ai_next) {
-			if (res->ai_addrlen > (socklen_t)sizeof(q.addr))
+			if (res->ai_addrlen > sizeof(q.addr))
 				continue;
 
 			/*
@@ -943,7 +932,7 @@ main(int argc, char *argv[])
 			 * address.
 			 */
 			local_address = find_by_address_family(local_addresses,
-				res->ai_family);
+							       res->ai_family);
 			if (local_addresses && !local_address) {
 				/* Continue with next remote address.  */
 				continue;
@@ -958,12 +947,14 @@ main(int argc, char *argv[])
 			}
 
 			/* Bind socket to local address, if required.  */
-			if (local_address && bind(state.s,
-				local_address->ai_addr,
-				local_address->ai_addrlen) < 0)
+			if (local_address
+			    && bind(state.s,
+				    local_address->ai_addr,
+				    local_address->ai_addrlen) < 0)
 			{
 				warning("cannot bind to %s: %s\n",
-					local_hostname, strerror(errno));
+					local_hostname,
+					strerror(errno));
 			}
 
 			if (connect(state.s, res->ai_addr, res->ai_addrlen) < 0)
@@ -1023,21 +1014,6 @@ main(int argc, char *argv[])
 	exit(XFER_FAIL);
 }
 
-void
-get_hostname_port_frm_str(char* arg, const char** hostname,
-	const char** port)
-{
-	/* parse -a src[@port] option */
-	char* delim = strchr(arg, '@');
-
-	if (delim) {
-		*delim = '\0';
-		*port = delim+1;
-	}
-	*hostname = arg;
-}
-
-
 int
 check_matching_address_family(struct addrinfo *a0, struct addrinfo *b0)
 {
