@@ -1,7 +1,7 @@
 /*
  * options.h -- nsd.conf options definitions and prototypes
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -10,7 +10,7 @@
 #ifndef OPTIONS_H
 #define OPTIONS_H
 
-#include "config.h"
+#include <config.h>
 #include <stdarg.h>
 #include "region-allocator.h"
 #include "rbtree.h"
@@ -34,13 +34,12 @@ struct nsd_options {
 	/* list of keys defined */
 	key_options_t* keys;
 	size_t numkeys;
-
+	
 	/* list of ip adresses to bind to (or NULL for all) */
 	ip_address_option_t* ip_addresses;
 
 	int debug_mode;
 	int verbosity;
-	int hide_version;
 	int ip4_only;
 	int ip6_only;
 	const char* database;
@@ -48,32 +47,17 @@ struct nsd_options {
 	const char* logfile;
 	int server_count;
 	int tcp_count;
-	int tcp_query_count;
-	int tcp_timeout;
-	size_t ipv4_edns_size;
-	size_t ipv6_edns_size;
 	const char* pidfile;
 	const char* port;
 	int statistics;
-	const char* zonestatsfile;
 	const char* chroot;
 	const char* username;
 	const char* zonesdir;
 	const char* difffile;
 	const char* xfrdfile;
-	const char* nsid;
 	int xfrd_reload_timeout;
-
-#ifdef RATELIMIT
-	/** number of buckets in rrl hashtable */
-	size_t rrl_size;
-	/** max qps for queries, 0 is nolimit */
-	size_t rrl_ratelimit;
-	/** max qps for whitelisted queries, 0 is nolimit */
-	size_t rrl_whitelist_ratelimit;
-#endif
-
-	region_type* region;
+	
+	region_type* region; 
 };
 
 struct ipaddress_option {
@@ -95,12 +79,6 @@ struct zone_options {
 	acl_options_t* request_xfr;
 	acl_options_t* notify;
 	acl_options_t* provide_xfr;
-	acl_options_t* outgoing_interface;
-#ifdef RATELIMIT
-	uint16_t rrl_whitelist; /* bitmap with rrl types */
-#endif
-	uint8_t allow_axfr_fallback;
-	uint8_t notify_retry;
 };
 
 union acl_addr_storage {
@@ -120,8 +98,6 @@ struct acl_options {
 
 	/* options */
 	uint8_t use_axfr_only;
-	uint8_t allow_udp;
-	time_t ixfr_disabled;
 
 	/* ip address range */
 	const char* ip_address_spec;
@@ -151,7 +127,9 @@ struct key_options {
 	const char* name;
 	const char* algorithm;
 	const char* secret;
+#ifdef TSIG
 	struct tsig_key* tsig_key;
+#endif
 };
 
 /*
@@ -169,7 +147,6 @@ struct config_parser_state {
 	acl_options_t* current_request_xfr;
 	acl_options_t* current_notify;
 	acl_options_t* current_provide_xfr;
-	acl_options_t* current_outgoing_interface;
 };
 
 extern config_parser_state_t* cfg_parser;
@@ -189,18 +166,14 @@ zone_options_t* zone_options_create(region_type* region);
 zone_options_t* zone_options_find(nsd_options_t* opt, const struct dname* apex);
 key_options_t* key_options_create(region_type* region);
 key_options_t* key_options_find(nsd_options_t* opt, const char* name);
-
-#if defined(HAVE_SSL)
 /* tsig must be inited, adds all keys in options to tsig. */
 void key_options_tsig_add(nsd_options_t* opt);
-#endif
 
-/* check acl list, acl number that matches if passed(0..),
+/* check acl list, acl number that matches if passed(0..), 
  * or failure (-1) if dropped */
 /* the reason why (the acl) is returned too (or NULL) */
-int acl_check_incoming(acl_options_t* acl, struct query* q,
+int acl_check_incoming(acl_options_t* acl, struct query* q, 
 	acl_options_t** reason);
-int acl_addr_matches_host(acl_options_t* acl, acl_options_t* host);
 int acl_addr_matches(acl_options_t* acl, struct query* q);
 int acl_key_matches(acl_options_t* acl, struct query* q);
 int acl_addr_match_mask(uint32_t* a, uint32_t* b, uint32_t* mask, size_t sz);
@@ -224,7 +197,5 @@ int parse_acl_is_ipv6(const char* p);
 int parse_acl_range_type(char* ip, char** mask);
 /* parses subnet mask, fills 0 mask as well */
 void parse_acl_range_subnet(char* p, void* addr, int maxbits);
-/* clean up options */
-void nsd_options_destroy(nsd_options_t* opt);
 
 #endif /* OPTIONS_H */
