@@ -1,13 +1,13 @@
 /*
  * query.c -- nsd(8) the resolver.
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -17,6 +17,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
+#include <netdb.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,14 +38,6 @@
 #include "nsec3.h"
 #include "tsig.h"
 
-/* [Bug #253] Adding unnecessary NS RRset may lead to undesired truncation.
- * This function determines if the final response packet needs the NS RRset
- * included. Currently, it will only return negative if QTYPE == DNSKEY|DS.
- * This way, resolvers won't fallback to TCP unnecessarily when priming
- * trust anchors.
- */
-static int answer_needs_ns(struct query  *query);
-
 static int add_rrset(struct query  *query,
 		     answer_type    *answer,
 		     rr_section_type section,
@@ -60,9 +53,9 @@ static void answer_authoritative(struct nsd	  *nsd,
 				 domain_type      *closest_encloser,
 				 const dname_type *qname);
 
-static void answer_lookup_zone(struct nsd *nsd, struct query *q,
-			       answer_type *answer, uint32_t domain_number,
-			       int exact, domain_type *closest_match,
+static void answer_lookup_zone(struct nsd *nsd, struct query *q, 
+			       answer_type *answer, uint32_t domain_number, 
+			       int exact, domain_type *closest_match, 
 			       domain_type *closest_encloser,
 			       const dname_type *qname);
 
@@ -77,7 +70,7 @@ query_put_dname_offset(struct query *q, domain_type *domain, uint16_t offset)
 		return;
 	if (q->compressed_dname_count >= MAX_COMPRESSED_DNAMES)
 		return;
-
+	
 	q->compressed_dname_offsets[domain->number] = offset;
 	q->compressed_dnames[q->compressed_dname_count] = domain;
 	++q->compressed_dname_count;
@@ -99,7 +92,7 @@ void
 query_clear_compression_tables(struct query *q)
 {
 	uint16_t i;
-
+	
 	for (i = 0; i < q->compressed_dname_count; ++i) {
 		assert(q->compressed_dnames);
 		q->compressed_dname_offsets[q->compressed_dnames[i]->number] = 0;
@@ -131,12 +124,12 @@ query_error (struct query *q, nsd_rc_type rcode)
 	if (rcode == NSD_RC_DISCARD) {
 		return QUERY_DISCARDED;
 	}
-
+	
 	buffer_clear(q->packet);
-
+	
 	QR_SET(q->packet);	   /* This is an answer.  */
 	RCODE_SET(q->packet, (int) rcode); /* Error code.  */
-
+	
 	/* Truncate the question as well... */
 	QDCOUNT_SET(q->packet, 0);
 	ANCOUNT_SET(q->packet, 0);
@@ -176,25 +169,27 @@ query_create(region_type *region, uint16_t *compressed_dname_offsets,
 	query->packet = buffer_create(region, QIOBUFSZ);
 	region_add_cleanup(region, query_cleanup, query);
 	query->compressed_dname_offsets_size = compressed_dname_size;
+#ifdef TSIG
 	tsig_create_record(&query->tsig, region);
 	query->tsig_prepare_it = 1;
 	query->tsig_update_it = 1;
 	query->tsig_sign_it = 1;
+#endif /* TSIG */
 	return query;
 }
 
-void
+void 
 query_reset(query_type *q, size_t maxlen, int is_tcp)
 {
-	/*
+	/* 
 	 * As long as less than 4Kb (region block size) has been used,
 	 * this call to free_all is free, the block is saved for re-use,
-	 * so no malloc() or free() calls are done.
+	 * so no malloc() or free() calls are done. 
 	 * at present use of the region is for:
 	 *   o query qname dname_type (255 max).
 	 *   o wildcard expansion domain_type (7*ptr+u32+2bytes)+(5*ptr nsec3)
 	 *   o wildcard expansion for additional section domain_type.
-	 *   o nsec3 hashed name(s) (3 dnames for a nonexist_proof,
+	 *   o nsec3 hashed name(s) (3 dnames for a nonexist_proof, 
 	 *     one proof per wildcard and for nx domain).
 	 */
 	region_free_all(q->region);
@@ -203,10 +198,12 @@ query_reset(query_type *q, size_t maxlen, int is_tcp)
 	q->reserved_space = 0;
 	buffer_clear(q->packet);
 	edns_init_record(&q->edns);
+#ifdef TSIG
 	tsig_init_record(&q->tsig, NULL, NULL);
 	q->tsig_prepare_it = 1;
 	q->tsig_update_it = 1;
 	q->tsig_sign_it = 1;
+#endif /* TSIG */
 	q->tcp = is_tcp;
 	q->qname = NULL;
 	q->qtype = 0;
@@ -236,12 +233,12 @@ query_get_tempdomain(struct query *q)
 		return 0;
 	q->number_temporary_domains ++;
 	memset(&d[q->number_temporary_domains-1], 0, sizeof(domain_type));
-	d[q->number_temporary_domains-1].number = q->compressed_dname_offsets_size +
+	d[q->number_temporary_domains-1].number = q->compressed_dname_offsets_size + 
 		q->number_temporary_domains - 1;
 	return &d[q->number_temporary_domains-1];
 }
 
-static void
+static void 
 query_addtxt(struct query  *q,
 	     const uint8_t *dname,
 	     uint16_t       klass,
@@ -252,7 +249,7 @@ query_addtxt(struct query  *q,
 	uint8_t len = (uint8_t) txt_length;
 
 	assert(txt_length <= UCHAR_MAX);
-
+	
 	/* Add the dname */
 	if (dname >= buffer_begin(q->packet)
 	    && dname <= buffer_current(q->packet))
@@ -301,31 +298,21 @@ process_query_section(query_type *query)
  * Return NSD_RC_FORMAT on failure, NSD_RC_OK on success.
  */
 static nsd_rc_type
-process_edns(nsd_type* nsd, struct query *q)
+process_edns(struct query *q)
 {
 	if (q->edns.status == EDNS_ERROR) {
-		/* The only error is VERSION not implemented */
 		return NSD_RC_FORMAT;
 	}
-
 	if (q->edns.status == EDNS_OK) {
 		/* Only care about UDP size larger than normal... */
 		if (!q->tcp && q->edns.maxlen > UDP_MAX_MESSAGE_LEN) {
-			size_t edns_size;
-#if defined(INET6)
-			if (q->addr.ss_family == AF_INET6) {
-				edns_size = nsd->ipv6_edns_size;
-			} else
-#endif
-			edns_size = nsd->ipv4_edns_size;
-
-			if (q->edns.maxlen < edns_size) {
+			if (q->edns.maxlen < EDNS_MAX_MESSAGE_LEN) {
 				q->maxlen = q->edns.maxlen;
 			} else {
-				q->maxlen = edns_size;
+				q->maxlen = EDNS_MAX_MESSAGE_LEN;
 			}
-
-#if defined(INET6) && !defined(IPV6_USE_MIN_MTU) && !defined(IPV6_MTU)
+			
+#if defined(INET6) && !defined(IPV6_USE_MIN_MTU)
 			/*
 			 * Use IPv6 minimum MTU to avoid sending
 			 * packets that are too large for some links.
@@ -339,7 +326,7 @@ process_edns(nsd_type* nsd, struct query *q)
 			}
 #endif
 		}
-
+		
 		/* Strip the OPT resource record off... */
 		buffer_set_position(q->packet, q->edns.position);
 		buffer_set_limit(q->packet, q->edns.position);
@@ -352,6 +339,7 @@ process_edns(nsd_type* nsd, struct query *q)
  * Processes TSIG.
  * Sets error when tsig does not verify on the query.
  */
+#ifdef TSIG
 static nsd_rc_type
 process_tsig(struct query* q)
 {
@@ -376,14 +364,15 @@ process_tsig(struct query* q)
 	}
 	return NSD_RC_OK;
 }
+#endif /* TSIG */
 
 /*
  * Check notify acl and forward to xfrd (or return an error).
  */
 static query_state_type
-answer_notify(struct nsd* nsd, struct query *query)
+answer_notify (struct nsd* nsd, struct query *query)
 {
-	int acl_num, acl_num_xfr;
+	int acl_num;
 	acl_options_t *why;
 	nsd_rc_type rc;
 
@@ -392,12 +381,13 @@ answer_notify(struct nsd* nsd, struct query *query)
 		dname_to_string(query->qname, NULL)));
 
 	zone_opt = zone_options_find(nsd->options, query->qname);
-	if(!zone_opt)
+	if(!zone_opt) 
 		return query_error(query, NSD_RC_NXDOMAIN);
-
+	
 	if(!nsd->this_child) /* we are in debug mode or something */
 		return query_error(query, NSD_RC_SERVFAIL);
-
+	
+#ifdef TSIG
 	if(!tsig_find_rr(&query->tsig, query->packet)) {
 		DEBUG(DEBUG_XFRD,2, (LOG_ERR, "bad tsig RR format"));
 		return query_error(query, NSD_RC_FORMAT);
@@ -405,24 +395,21 @@ answer_notify(struct nsd* nsd, struct query *query)
 	rc = process_tsig(query);
 	if(rc != NSD_RC_OK)
 		return query_error(query, rc);
+#endif /* TSIG */
 
 	/* check if it passes acl */
 	if((acl_num = acl_check_incoming(zone_opt->allow_notify, query,
 		&why)) != -1)
 	{
-		/* Find priority candidate for request XFR. -1 if no match */
-		acl_num_xfr = acl_check_incoming(
-			zone_opt->request_xfr, query, NULL);
 		sig_atomic_t mode = NSD_PASS_TO_XFRD;
 		int s = nsd->this_child->parent_fd;
 		uint16_t sz;
 		uint32_t acl_send = htonl(acl_num);
-		uint32_t acl_xfr = htonl(acl_num_xfr);
 		size_t pos;
 		assert(why);
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "got notify %s passed acl %s %s",
 			dname_to_string(query->qname, NULL),
-			why->ip_address_spec,
+			why->ip_address_spec, 
 			why->nokey?"NOKEY":
 			(why->blocked?"BLOCKED":why->key_name)));
 		sz = buffer_limit(query->packet);
@@ -431,12 +418,11 @@ answer_notify(struct nsd* nsd, struct query *query)
 		/* forward to xfrd for processing
 		   Note. Blocking IPC I/O, but acl is OK. */
 		sz = htons(sz);
-		if(!write_socket(s, &mode, sizeof(mode)) ||
+		if(!write_socket(s, &mode, sizeof(mode)) || 
 			!write_socket(s, &sz, sizeof(sz)) ||
 			!write_socket(s, buffer_begin(query->packet),
 				buffer_limit(query->packet)) ||
-			!write_socket(s, &acl_send, sizeof(acl_send)) ||
-			!write_socket(s, &acl_xfr, sizeof(acl_xfr))) {
+			!write_socket(s, &acl_send, sizeof(acl_send))) {
 			log_msg(LOG_ERR, "error in IPC notify server2main, %s",
 				strerror(errno));
 			return query_error(query, NSD_RC_SERVFAIL);
@@ -453,25 +439,14 @@ answer_notify(struct nsd* nsd, struct query *query)
 		pos = buffer_position(query->packet);
 		buffer_clear(query->packet);
 		buffer_set_position(query->packet, pos);
-		VERBOSITY(2, (LOG_INFO, "Notify received and accepted, forward to xfrd"));
+		VERBOSITY(2, (LOG_INFO, "Notify received and accepted, forward to xfrd")); 
 		/* tsig is added in add_additional later (if needed) */
 		return QUERY_PROCESSED;
 	}
-
-	if (verbosity > 1) {
-		char address[128];
-		if (addr2ip(query->addr, address, sizeof(address))) {
-			DEBUG(DEBUG_XFRD,1, (LOG_INFO, "addr2ip failed"));
-			strlcpy(address, "[unknown]", sizeof(address));
-		}
-
-		VERBOSITY(1, (LOG_INFO, "notify for zone %s from client %s refused, %s%s",
+	VERBOSITY(1, (LOG_INFO, "got notify for zone: %s; Refused by acl: %s %s",
 			dname_to_string(query->qname, NULL),
-			address,
-			why?why->key_name:"no acl matches",
+			why?why->key_name:"no acl matches", 
 			why?why->ip_address_spec:"."));
-	}
-
 	return query_error(query, NSD_RC_REFUSE);
 }
 
@@ -487,7 +462,7 @@ answer_chaos(struct nsd *nsd, query_type *q)
 	case TYPE_ANY:
 	case TYPE_TXT:
 		if ((q->qname->name_size == 11
-		     && memcmp(dname_name(q->qname), "\002id\006server", 11) == 0) ||
+		     && memcmp(dname_name(q->qname), "\002id\006server", 11) == 0) || 
 		    (q->qname->name_size ==  15
 		     && memcmp(dname_name(q->qname), "\010hostname\004bind", 15) == 0))
 		{
@@ -503,17 +478,13 @@ answer_chaos(struct nsd *nsd, query_type *q)
 			   (q->qname->name_size == 14
 			    && memcmp(dname_name(q->qname), "\007version\004bind", 14) == 0))
 		{
-			if(!nsd->options->hide_version) {
-				/* Add version */
-				query_addtxt(q,
+			/* Add version */
+			query_addtxt(q,
 				     buffer_begin(q->packet) + QHEADERSZ,
 				     CLASS_CH,
 				     0,
 				     nsd->version);
-				ANCOUNT_SET(q->packet, ANCOUNT(q->packet) + 1);
-			} else {
-				RCODE_SET(q->packet, RCODE_REFUSE);
-			}
+			ANCOUNT_SET(q->packet, ANCOUNT(q->packet) + 1);
 		}
 		break;
 	default:
@@ -541,7 +512,7 @@ find_covering_nsec(domain_type *closest_match,
 
 	/* loop away temporary created domains. For real ones it is &RBTREE_NULL */
 	while (closest_match->node.parent == NULL)
-		closest_match = closest_match->parent;
+		closest_match = closest_match->parent; 
 	while (closest_match) {
 		*nsec_rrset = domain_find_rrset(closest_match, zone, TYPE_NSEC);
 		if (*nsec_rrset) {
@@ -583,22 +554,22 @@ add_additional_rrsets(struct query *query, answer_type *answer,
 		      int allow_glue, struct additional_rr_types types[])
 {
 	size_t i;
-
+	
 	assert(query);
 	assert(answer);
 	assert(master_rrset);
 	assert(rdata_atom_is_domain(rrset_rrtype(master_rrset), rdata_index));
-
+	
 	for (i = 0; i < master_rrset->rr_count; ++i) {
 		int j;
 		domain_type *additional = rdata_atom_domain(master_rrset->rrs[i].rdatas[rdata_index]);
 		domain_type *match = additional;
-
+		
 		assert(additional);
 
 		if (!allow_glue && domain_is_glue(match, query->zone))
 			continue;
-
+		
 		/*
 		 * Check to see if we need to generate the dependent
 		 * based on a wildcard domain.
@@ -630,15 +601,6 @@ add_additional_rrsets(struct query *query, answer_type *answer,
 	}
 }
 
-static int
-answer_needs_ns(struct query* query)
-{
-	assert(query);
-	/* Currently, only troublesome for DNSKEY and DS,
-         * cuz their RRSETs are quite large. */
-	return (query->qtype != TYPE_DNSKEY && query->qtype != TYPE_DS);
-}
-
 static int
 add_rrset(struct query   *query,
 	  answer_type    *answer,
@@ -647,13 +609,13 @@ add_rrset(struct query   *query,
 	  rrset_type     *rrset)
 {
 	int result;
-
+	
 	assert(query);
 	assert(answer);
 	assert(owner);
 	assert(rrset);
 	assert(rrset_rrclass(rrset) == CLASS_IN);
-
+	
 	result = answer_add_rrset(answer, section, owner, rrset);
 	switch (rrset_rrtype(rrset)) {
 	case TYPE_NS:
@@ -686,7 +648,7 @@ add_rrset(struct query   *query,
    DNAME rr is from src to dest.
    closest encloser encloses the to_name. */
 static uint32_t
-query_synthesize_cname(struct query* q, struct answer* answer, const dname_type* from_name,
+query_synthesize_cname(struct query* q, struct answer* answer, const dname_type* from_name, 
 	const dname_type* to_name, domain_type* src, domain_type* to_closest_encloser,
 	domain_type** to_closest_match)
 {
@@ -705,7 +667,7 @@ query_synthesize_cname(struct query* q, struct answer* answer, const dname_type*
 	for(i=0; i < from_name->label_count - domain_dname(src)->label_count; i++)
 	{
 		domain_type* newdom = query_get_tempdomain(q);
-		if(!newdom)
+		if(!newdom) 
 			return 0;
 		newdom->is_existing = 1;
 		newdom->parent = lastparent;
@@ -724,11 +686,11 @@ query_synthesize_cname(struct query* q, struct answer* answer, const dname_type*
 
 	/* allocate dest part */
 	lastparent = to_closest_encloser;
-	for(i=0; i < to_name->label_count - domain_dname(to_closest_encloser)->label_count;
+	for(i=0; i < to_name->label_count - domain_dname(to_closest_encloser)->label_count; 
 		i++)
 	{
 		domain_type* newdom = query_get_tempdomain(q);
-		if(!newdom)
+		if(!newdom) 
 			return 0;
 		newdom->is_existing = 0;
 		newdom->parent = lastparent;
@@ -777,13 +739,8 @@ answer_delegation(query_type *query, answer_type *answer)
 	assert(answer);
 	assert(query->delegation_domain);
 	assert(query->delegation_rrset);
-
-	if (query->cname_count == 0) {
-		AA_CLR(query->packet);
-	} else {
-		AA_SET(query->packet);
-	}
-
+	
+	AA_CLR(query->packet);
 	add_rrset(query,
 		  answer,
 		  AUTHORITY_SECTION,
@@ -814,7 +771,7 @@ static void
 answer_soa(struct query *query, answer_type *answer)
 {
 	query->domain = query->zone->apex;
-
+	
 	if (query->qclass != CLASS_ANY) {
 		add_rrset(query, answer,
 			  AUTHORITY_SECTION,
@@ -839,11 +796,11 @@ answer_nodata(struct query *query, answer_type *answer, domain_type *original)
 	if (query->cname_count == 0) {
 		answer_soa(query, answer);
 	}
-
+	
 #ifdef NSEC3
 	if (query->edns.dnssec_ok && query->zone->nsec3_soa_rr) {
 		nsec3_answer_nodata(query, answer, original);
-	} else
+	} else 
 #endif
 	if (query->edns.dnssec_ok && zone_is_secure(query->zone)) {
 		domain_type *nsec_domain;
@@ -875,7 +832,7 @@ answer_domain(struct nsd* nsd, struct query *q, answer_type *answer,
 	      domain_type *domain, domain_type *original)
 {
 	rrset_type *rrset;
-
+	
 	if (q->qtype == TYPE_ANY) {
 		int added = 0;
 		for (rrset = domain_find_any_rrset(domain, q->zone); rrset; rrset = rrset->next) {
@@ -923,28 +880,25 @@ answer_domain(struct nsd* nsd, struct query *q, answer_type *answer,
 			domain_type *closest_encloser = closest_match;
 			zone_type* origzone = q->zone;
 			++q->cname_count;
-
+				
 			while (!closest_encloser->is_existing)
 				closest_encloser = closest_encloser->parent;
-
+				
 			answer_lookup_zone(nsd, q, answer, closest_match->number,
 					     closest_match == closest_encloser,
 					     closest_match, closest_encloser,
 					     domain_dname(closest_match));
 			q->zone = origzone;
 		}
-		/* example 6.2.7 shows no NS-set from zone in auth (RFC1034) */
-		q->domain = domain;
-		return;
 	} else {
 		answer_nodata(q, answer, original);
 		return;
 	}
 
 	q->domain = domain;
-
-	if (q->qclass != CLASS_ANY && q->zone->ns_rrset && answer_needs_ns(q)) {
-		add_rrset(q, answer, OPTIONAL_AUTHORITY_SECTION, q->zone->apex,
+	
+	if (q->qclass != CLASS_ANY && q->zone->ns_rrset) {
+		add_rrset(q, answer, AUTHORITY_SECTION, q->zone->apex,
 			  q->zone->ns_rrset);
 	}
 }
@@ -972,7 +926,7 @@ answer_authoritative(struct nsd   *nsd,
 	domain_type *match;
 	domain_type *original = closest_match;
 	rrset_type *rrset;
-
+	
 #ifdef NSEC3
 	if(exact && domain_has_only_NSEC3(closest_match, q->zone)) {
 		exact = 0; /* pretend it does not exist */
@@ -992,15 +946,15 @@ answer_authoritative(struct nsd   *nsd,
 		if(domain_number != 0) /* we followed CNAMEs or DNAMEs */
 			name = domain_dname(closest_match);
 		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "expanding DNAME for q=%s", dname_to_string(name, NULL)));
-		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->src is %s",
+		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->src is %s", 
 			dname_to_string(domain_dname(closest_encloser), NULL)));
-		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->dest is %s",
+		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->dest is %s", 
 			dname_to_string(domain_dname(dest), NULL)));
 		/* if the DNAME set is not added we have a loop, do not follow */
 		added = add_rrset(q, answer, ANSWER_SECTION, closest_encloser, rrset);
 		if(added) {
 			domain_type* src = closest_encloser;
-			const dname_type* newname = dname_replace(q->region, name,
+			const dname_type* newname = dname_replace(q->region, name, 
 				domain_dname(src), domain_dname(dest));
 			uint32_t newnum = 0;
 			zone_type* origzone = q->zone;
@@ -1013,7 +967,7 @@ answer_authoritative(struct nsd   *nsd,
 			/* follow the DNAME */
 			exact = namedb_lookup(nsd->db, newname, &closest_match, &closest_encloser);
 			/* synthesize CNAME record */
-			newnum = query_synthesize_cname(q, answer, name, newname,
+			newnum = query_synthesize_cname(q, answer, name, newname, 
 				src, closest_encloser, &closest_match);
 			if(!newnum) {
 				/* could not synthesize the CNAME. */
@@ -1024,7 +978,7 @@ answer_authoritative(struct nsd   *nsd,
 			while (closest_encloser && !closest_encloser->is_existing)
 				closest_encloser = closest_encloser->parent;
 			answer_lookup_zone(nsd, q, answer, newnum,
-				closest_match == closest_encloser,
+				closest_match == closest_encloser, 
 				closest_match, closest_encloser, newname);
 			q->zone = origzone;
 		}
@@ -1045,17 +999,12 @@ answer_authoritative(struct nsd   *nsd,
 		match->rrsets = wildcard_child->rrsets;
 		match->is_existing = wildcard_child->is_existing;
 #ifdef NSEC3
-		match->nsec3_cover = wildcard_child->nsec3_cover;
-#ifdef FULL_PREHASH
 		match->nsec3_is_exact = wildcard_child->nsec3_is_exact;
+		match->nsec3_cover = wildcard_child->nsec3_cover;
 		match->nsec3_wcard_child_cover = wildcard_child->nsec3_wcard_child_cover;
 		match->nsec3_ds_parent_is_exact = wildcard_child->nsec3_ds_parent_is_exact;
 		match->nsec3_ds_parent_cover = wildcard_child->nsec3_ds_parent_cover;
-#endif
-		if (q->edns.dnssec_ok && q->zone->nsec3_soa_rr) {
-			/* Only add nsec3 wildcard data when do bit is set */
-			nsec3_answer_wildcard(q, answer, wildcard_child, nsd->db, qname);
-		}
+		nsec3_answer_wildcard(q, answer, wildcard_child, nsd->db, qname);
 #endif
 
 		/*
@@ -1072,15 +1021,15 @@ answer_authoritative(struct nsd   *nsd,
 	/* Authorative zone.  */
 #ifdef NSEC3
 	if (q->edns.dnssec_ok && q->zone->nsec3_soa_rr) {
-		nsec3_answer_authoritative(&match, q, answer,
+		nsec3_answer_authoritative(&match, q, answer, 
 			closest_encloser, nsd->db, qname);
-	} else
+	} else 
 #endif
 	if (q->edns.dnssec_ok && zone_is_secure(q->zone)) {
 		if (match != closest_encloser) {
 			domain_type *nsec_domain;
 			rrset_type *nsec_rrset;
-
+			
 			/*
 			 * No match found or generated from wildcard,
 			 * include NSEC record.
@@ -1104,7 +1053,7 @@ answer_authoritative(struct nsd   *nsd,
 			}
 		}
 	}
-
+	
 #ifdef NSEC3
 	if (RCODE(q->packet)!=RCODE_OK) {
 		return; /* nsec3 collision failure */
@@ -1122,9 +1071,9 @@ answer_authoritative(struct nsd   *nsd,
  */
 static void
 answer_lookup_zone(struct nsd *nsd, struct query *q, answer_type *answer,
-	uint32_t domain_number, int exact, domain_type *closest_match,
+	uint32_t domain_number, int exact, domain_type *closest_match, 
 	domain_type *closest_encloser, const dname_type *qname)
-{
+{	
 	q->zone = domain_find_zone(closest_encloser);
 	if (!q->zone) {
 		if(q->cname_count == 0)
@@ -1210,20 +1159,14 @@ answer_query(struct nsd *nsd, struct query *q)
 	}
 
 	q->domain = closest_encloser;
-	answer_lookup_zone(nsd, q, &answer, 0, exact, closest_match,
+	answer_lookup_zone(nsd, q, &answer, 0, exact, closest_match, 
 		closest_encloser, q->qname);
 
-#ifdef USE_ZONE_STATS
-	if (q->zone) {
-		ZTATUP2(q->zone, opcode, q->opcode);
-		ZTATUP2(q->zone, qtype, q->qtype);
-		ZTATUP2(q->zone, opcode, q->qclass);
-	}
-#endif
-
 	offset = dname_label_offsets(q->qname)[domain_dname(closest_encloser)->label_count - 1] + QHEADERSZ;
 	query_add_compression_domain(q, closest_encloser, offset);
+
 	encode_answer(q, &answer);
+
 	query_clear_compression_tables(q);
 }
 
@@ -1231,19 +1174,21 @@ void
 query_prepare_response(query_type *q)
 {
 	uint16_t flags;
-
+	
 	/*
 	 * Preserve the data up-to the current packet's limit.
 	 */
 	buffer_set_position(q->packet, buffer_limit(q->packet));
 	buffer_set_limit(q->packet, buffer_capacity(q->packet));
-
+	
 	/*
 	 * Reserve space for the EDNS records if required.
 	 */
 	q->reserved_space = edns_reserved_space(&q->edns);
+#ifdef TSIG
 	q->reserved_space += tsig_reserved_space(&q->tsig);
-
+#endif /* TSIG */
+	
 	/* Update the flags.  */
 	flags = FLAGS(q->packet);
 	flags &= 0x0100U;	/* Preserve the RD flag.  */
@@ -1266,7 +1211,7 @@ query_process(query_type *q, nsd_type *nsd)
 
 	/* Sanity checks */
 	if (buffer_limit(q->packet) < QHEADERSZ) {
-		/* packet too small to contain DNS header.
+		/* packet too small to contain DNS header. 
 		Now packet investigation macros will work without problems. */
 		return QUERY_DISCARDED;
 	}
@@ -1275,7 +1220,7 @@ query_process(query_type *q, nsd_type *nsd)
 		return QUERY_DISCARDED;
 	}
 
-	if (RCODE(q->packet) != RCODE_OK || !process_query_section(q)) {
+	if(!process_query_section(q)) {
 		return query_formerr(q);
 	}
 
@@ -1293,26 +1238,26 @@ query_process(query_type *q, nsd_type *nsd)
 	}
 
 	/* Dont bother to answer more than one question at once... */
-	if (QDCOUNT(q->packet) != 1) {
+	if (QDCOUNT(q->packet) != 1 || TC(q->packet)) {
 		FLAGS_SET(q->packet, 0);
 		return query_formerr(q);
 	}
-	/* Ignore settings of flags */
 
-	/* Dont allow any records in the answer or authority section...
+	/* Dont allow any records in the answer or authority section... 
 	   except for IXFR queries. */
-	if (ANCOUNT(q->packet) != 0 ||
+	if (ANCOUNT(q->packet) != 0 || 
 		(q->qtype!=TYPE_IXFR && NSCOUNT(q->packet) != 0)) {
 		return query_formerr(q);
 	}
 	if(q->qtype==TYPE_IXFR && NSCOUNT(q->packet) > 0) {
 		int i; /* skip ixfr soa information data here */
-		for(i=0; i< NSCOUNT(q->packet); i++)
+		for(i=0; i<NSCOUNT(q->packet); i++)
 			if(!packet_skip_rr(q->packet, 0))
 				return query_formerr(q);
 	}
 
 	arcount = ARCOUNT(q->packet);
+#ifdef TSIG
 	if (arcount > 0) {
 		/* see if tsig is before edns record */
 		if (!tsig_parse_rr(&q->tsig, q->packet))
@@ -1320,10 +1265,12 @@ query_process(query_type *q, nsd_type *nsd)
 		if(q->tsig.status != TSIG_NOT_PRESENT)
 			--arcount;
 	}
+#endif /* TSIG */
 	if (arcount > 0) {
 		if (edns_parse_record(&q->edns, q->packet))
 			--arcount;
 	}
+#ifdef TSIG
 	if (arcount > 0 && q->tsig.status == TSIG_NOT_PRESENT) {
 		/* see if tsig is after the edns record */
 		if (!tsig_parse_rr(&q->tsig, q->packet))
@@ -1331,6 +1278,7 @@ query_process(query_type *q, nsd_type *nsd)
 		if(q->tsig.status != TSIG_NOT_PRESENT)
 			--arcount;
 	}
+#endif /* TSIG */
 	if (arcount > 0) {
 		return query_formerr(q);
 	}
@@ -1344,22 +1292,20 @@ query_process(query_type *q, nsd_type *nsd)
 #endif
 	/* Remove trailing garbage.  */
 	buffer_set_limit(q->packet, buffer_position(q->packet));
-
+	
+#ifdef TSIG
 	rc = process_tsig(q);
 	if (rc != NSD_RC_OK) {
 		return query_error(q, rc);
 	}
-	rc = process_edns(nsd, q);
+#endif /* TSIG */
+	rc = process_edns(q);
 	if (rc != NSD_RC_OK) {
-		/* We should not return FORMERR, but BADVERS (=16).
-		 * BADVERS is created with Ext. RCODE, followed by RCODE.
-		 * Ext. RCODE is set to 1, RCODE must be 0 (getting 0x10 = 16).
-		 * Thus RCODE = NOERROR = NSD_RC_OK. */
-		return query_error(q, NSD_RC_OK);
+		return query_error(q, rc);
 	}
 
 	query_prepare_response(q);
-
+	
 	if (q->qclass != CLASS_IN && q->qclass != CLASS_ANY) {
 		if (q->qclass == CLASS_CH) {
 			return answer_chaos(nsd, q);
@@ -1387,16 +1333,13 @@ query_add_optional(query_type *q, nsd_type *nsd)
 		edns = &nsd->edns_ipv6;
 	}
 #endif
-	if (RCODE(q->packet) == RCODE_FORMAT) {
-		return;
-	}
 	switch (q->edns.status) {
 	case EDNS_NOT_PRESENT:
 		break;
 	case EDNS_OK:
-		if (q->edns.dnssec_ok)	edns->ok[7] = 0x80;
-		else			edns->ok[7] = 0x00;
 		buffer_write(q->packet, edns->ok, OPT_LEN);
+		/* check if nsid data should be written */
+#ifdef NSID
 		if (nsd->nsid_len > 0 && q->edns.nsid == 1 &&
 				!query_overflow_nsid(q, nsd->nsid_len)) {
 			/* rdata length */
@@ -1409,36 +1352,30 @@ query_add_optional(query_type *q, nsd_type *nsd)
 			/* fill with NULLs */
 			buffer_write(q->packet, edns->rdata_none, OPT_RDATA);
 		}
+#else
+		buffer_write(q->packet, edns->rdata_none, OPT_RDATA);
+#endif /* NSID */
+		
 		ARCOUNT_SET(q->packet, ARCOUNT(q->packet) + 1);
+
 		STATUP(nsd, edns);
-#ifdef USE_ZONE_STATS
-		if (q->zone) {
-			ZTATUP(q->zone, edns);
-		}
-#endif
 		break;
 	case EDNS_ERROR:
-		if (q->edns.dnssec_ok)	edns->error[7] = 0x80;
-		else			edns->error[7] = 0x00;
 		buffer_write(q->packet, edns->error, OPT_LEN);
-		buffer_write(q->packet, edns->rdata_none, OPT_RDATA);
 		ARCOUNT_SET(q->packet, ARCOUNT(q->packet) + 1);
+
 		STATUP(nsd, ednserr);
-#ifdef USE_ZONE_STATS
-		if (q->zone) {
-			ZTATUP(q->zone, ednserr);
-		}
-#endif
 		break;
 	}
 
+#ifdef TSIG
 	if (q->tsig.status != TSIG_NOT_PRESENT) {
-		if (q->tsig.status == TSIG_ERROR ||
+		if (q->tsig.status == TSIG_ERROR || 
 			q->tsig.error_code != TSIG_ERROR_NOERROR) {
 			tsig_error_reply(&q->tsig);
 			tsig_append_rr(&q->tsig, q->packet);
 			ARCOUNT_SET(q->packet, ARCOUNT(q->packet) + 1);
-		} else if(q->tsig.status == TSIG_OK &&
+		} else if(q->tsig.status == TSIG_OK && 
 			q->tsig.error_code == TSIG_ERROR_NOERROR)
 		{
 			if(q->tsig_prepare_it)
@@ -1452,4 +1389,5 @@ query_add_optional(query_type *q, nsd_type *nsd)
 			}
 		}
 	}
+#endif /* TSIG */
 }
