@@ -1,7 +1,7 @@
 /*
  * xfrd-notify.h - notify sending routines.
  *
- * Copyright (c) 2006-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -32,7 +32,9 @@ struct notify_zone_t {
 	const dname_type* apex;
 	const char* apex_str;
 
+#ifdef TSIG
         tsig_record_type notify_tsig; /* tsig state for notify */
+#endif
 	struct zone_options* options;
 	struct xfrd_soa *current_soa; /* current SOA in NSD */
 
@@ -52,16 +54,16 @@ struct notify_zone_t {
 
 /* initialise outgoing notifies */
 void init_notify_send(rbtree_t* tree, netio_type* netio, region_type* region,
-        const dname_type* apex, struct zone_options* options,
+        const dname_type* apex, struct zone_options* options, 
 	struct zone* dbzone);
 
 /* send notifications to all in the notify list */
 void xfrd_send_notify(rbtree_t* tree, const struct dname* apex,
 	struct xfrd_soa* new_soa);
 
-/* handle soa update notify for a master zone. newsoa can be NULL.
+/* handle soa update notify for a master zone. newsoa can be NULL. 
    Makes sure that the soa (serial) has changed. Or drops notify. */
-void notify_handle_master_zone_soainfo(rbtree_t* tree,
+void notify_handle_master_zone_soainfo(rbtree_t* tree, 
 	const dname_type* apex, struct xfrd_soa* new_soa);
 
 /* close fds in use for notification sending */
