@@ -1,7 +1,7 @@
 /*
  * zonec.c -- zone compiler.
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -37,12 +37,10 @@
 #include "rdata.h"
 #include "region-allocator.h"
 #include "util.h"
+#include "zparser.h"
 #include "options.h"
 #include "nsec3.h"
 
-#define ILNP_MAXDIGITS 4
-#define ILNP_NUMGROUPS 4
-
 const dname_type *error_dname;
 domain_type *error_domain;
 
@@ -83,7 +81,7 @@ alloc_rdata_init(region_type *region, const void *data, size_t size)
 	return result;
 }
 
-/*
+/* 
  * These are parser function for generic zone file stuff.
  */
 uint16_t *
@@ -93,7 +91,7 @@ zparser_conv_hex(region_type *region, const char *hex, size_t len)
 	uint16_t *r = NULL;
 	uint8_t *t;
 	int i;
-
+	
 	if (len % 2 != 0) {
 		zc_error_prev_line("number of hex digits must be a multiple of 2");
 	} else if (len > MAX_RDLENGTH * 2) {
@@ -103,12 +101,12 @@ zparser_conv_hex(region_type *region, const char *hex, size_t len)
 		/* the length part */
 		r = alloc_rdata(region, len/2);
 		t = (uint8_t *)(r + 1);
-
+    
 		/* Now process octet by octet... */
 		while (*hex) {
 			*t = 0;
 			for (i = 16; i >= 1; i -= 15) {
-				if (isxdigit((int)*hex)) {
+				if (isxdigit(*hex)) {
 					*t += hexdigit_to_int(*hex) * i;
 				} else {
 					zc_error_prev_line(
@@ -137,19 +135,19 @@ zparser_conv_hex_length(region_type *region, const char *hex, size_t len)
 		zc_error_prev_line("hex data exceeds 255 bytes");
 	} else {
 		uint8_t *l;
-
+		
 		/* the length part */
 		r = alloc_rdata(region, len/2+1);
 		t = (uint8_t *)(r + 1);
-
+		
 		l = t++;
 		*l = '\0';
-
+		
 		/* Now process octet by octet... */
 		while (*hex) {
 			*t = 0;
 			for (i = 16; i >= 1; i -= 15) {
-				if (isxdigit((int)*hex)) {
+				if (isxdigit(*hex)) {
 					*t += hexdigit_to_int(*hex) * i;
 				} else {
 					zc_error_prev_line(
@@ -214,7 +212,7 @@ zparser_conv_services(region_type *region, const char *protostr,
 	for (word = strtok(servicestr, sep); word; word = strtok(NULL, sep)) {
 		struct servent *service;
 		int port;
-
+		
 		service = getservbyname(word, proto->p_name);
 		if (service) {
 			/* Note: ntohs not ntohl!  Strange but true.  */
@@ -242,24 +240,7 @@ zparser_conv_services(region_type *region, const char *protostr,
 	p = (uint8_t *) (r + 1);
 	*p = proto->p_proto;
 	memcpy(p + 1, bitmap, *r);
-
-	return r;
-}
-
-uint16_t *
-zparser_conv_serial(region_type *region, const char *serialstr)
-{
-	uint16_t *r = NULL;
-	uint32_t serial;
-	const char *t;
-
-	serial = strtoserial(serialstr, &t);
-	if (*t != '\0') {
-		zc_error_prev_line("serial is expected");
-	} else {
-		serial = htonl(serial);
-		r = alloc_rdata_init(region, &serial, sizeof(serial));
-	}
+	
 	return r;
 }
 
@@ -269,10 +250,10 @@ zparser_conv_period(region_type *region, const char *periodstr)
 	/* convert a time period (think TTL's) to wireformat) */
 	uint16_t *r = NULL;
 	uint32_t period;
-	const char *end;
+	const char *end; 
 
 	/* Allocate required space... */
-	period = strtottl(periodstr, &end);
+	period = (uint32_t) strtottl(periodstr, &end);
 	if (*end != '\0') {
 		zc_error_prev_line("time period is expected");
 	} else {
@@ -288,7 +269,7 @@ zparser_conv_short(region_type *region, const char *text)
 	uint16_t *r = NULL;
 	uint16_t value;
 	char *end;
-
+   
 	value = htons((uint16_t) strtol(text, &end, 10));
 	if (*end != '\0') {
 		zc_error_prev_line("integer value is expected");
@@ -304,7 +285,7 @@ zparser_conv_byte(region_type *region, const char *text)
 	uint16_t *r = NULL;
 	uint8_t value;
 	char *end;
-
+   
 	value = (uint8_t) strtol(text, &end, 10);
 	if (*end != '\0') {
 		zc_error_prev_line("integer value is expected");
@@ -319,7 +300,7 @@ zparser_conv_algorithm(region_type *region, const char *text)
 {
 	const lookup_table_type *alg;
 	uint8_t id;
-
+	
 	alg = lookup_by_name(dns_algorithms, text);
 	if (alg) {
 		id = (uint8_t) alg->id;
@@ -341,7 +322,7 @@ zparser_conv_certificate_type(region_type *region, const char *text)
 	/* convert a algoritm string to integer */
 	const lookup_table_type *type;
 	uint16_t id;
-
+	
 	type = lookup_by_name(dns_certificate_types, text);
 	if (type) {
 		id = htons((uint16_t) type->id);
@@ -385,104 +366,21 @@ zparser_conv_aaaa(region_type *region, const char *text)
 	return r;
 }
 
-
-uint16_t *
-zparser_conv_ilnp64(region_type *region, const char *text)
-{
-	uint16_t *r = NULL;
-	int ngroups, num;
-	unsigned long hex;
-	const char *ch;
-	int c;
-	char digits[ILNP_MAXDIGITS+1];
-	unsigned int ui[ILNP_NUMGROUPS];
-	uint16_t a[ILNP_NUMGROUPS];
-
-	ngroups = 1; /* Always at least one group */
-	num = 0;
-	for (ch = text; *ch != '\0'; ch++) {
-		if (*ch == ':') {
-			if (num <= 0) {
-				zc_error_prev_line("ilnp64: empty group of "
-					"digits is not allowed");
-				return NULL;
-			}
-			digits[num] = '\0';
-			hex = (unsigned long) strtol(digits, NULL, 16);
-			num = 0;
-			ui[ngroups - 1] = hex;
-			if (ngroups >= ILNP_NUMGROUPS) {
-				zc_error_prev_line("ilnp64: more than %d groups "
-					"of digits", ILNP_NUMGROUPS);
-				return NULL;
-			}
-			ngroups++;
-		} else {
-			/* Our grammar is stricter than the one accepted by
-			 * strtol. */
-			c = (int) *ch;
-			if (!isxdigit(c)) {
-				zc_error_prev_line("ilnp64: invalid "
-					"(non-hexadecimal) character %c", c);
-				return NULL;
-			}
-			if (num >= ILNP_MAXDIGITS) {
-				zc_error_prev_line("ilnp64: more than %d digits "
-					"in a group", ILNP_MAXDIGITS);
-				return NULL;
-			}
-			digits[num++] = *ch;
-		}
-	}
-	if (num <= 0) {
-		zc_error_prev_line("ilnp64: empty group of digits is not "
-			"allowed");
-		return NULL;
-	}
-	digits[num] = '\0';
-	hex = (unsigned long) strtol(digits, NULL, 16);
-	ui[ngroups - 1] = hex;
-	if (ngroups < 4) {
-		zc_error_prev_line("ilnp64: less than %d groups of digits",
-			ILNP_NUMGROUPS);
-		return NULL;
-	}
-
-	a[0] = htons(ui[0]);
-	a[1] = htons(ui[1]);
-	a[2] = htons(ui[2]);
-	a[3] = htons(ui[3]);
-	r = alloc_rdata_init(region, a, sizeof(a));
-	return r;
-}
-
 uint16_t *
 zparser_conv_text(region_type *region, const char *text, size_t len)
 {
 	uint16_t *r = NULL;
-	uint8_t *p;
-
+	
 	if (len > 255) {
 		zc_error_prev_line("text string is longer than 255 characters,"
 				   " try splitting it into multiple parts");
-		len = 255;
+	} else {
+		uint8_t *p;
+		r = alloc_rdata(region, len + 1);
+		p = (uint8_t *) (r + 1);
+		*p = len;
+		memcpy(p + 1, text, len);
 	}
-	r = alloc_rdata(region, len + 1);
-	p = (uint8_t *) (r + 1);
-	*p = len;
-	memcpy(p + 1, text, len);
-	return r;
-}
-
-uint16_t *
-zparser_conv_dns_name(region_type *region, const uint8_t* name, size_t len)
-{
-	uint16_t* r = NULL;
-	uint8_t* p = NULL;
-	r = alloc_rdata(region, len);
-	p = (uint8_t *) (r + 1);
-	memcpy(p, name, len);
-
 	return r;
 }
 
@@ -581,7 +479,7 @@ zparser_conv_nsec(region_type *region,
 
 	window_max = 1 + (nsec_highest_rcode / 256);
 
-	/* used[i] is the i-th window included in the nsec
+	/* used[i] is the i-th window included in the nsec 
 	 * size[used[0]] is the size of window 0
 	 */
 
@@ -604,7 +502,7 @@ zparser_conv_nsec(region_type *region,
 	for (i = 0; i < window_count; ++i) {
 		total_size += sizeof(uint16_t) + size[used[i]];
 	}
-
+	
 	r = alloc_rdata(region, total_size);
 	ptr = (uint8_t *) (r + 1);
 
@@ -649,7 +547,7 @@ static unsigned int poweroften[10] = {1, 10, 100, 1000, 10000, 100000,
  * Sets the given pointer to the last used character.
  *
  */
-static uint8_t
+static uint8_t 
 precsize_aton (char *cp, char **endptr)
 {
 	unsigned int mval = 0, cmval = 0;
@@ -657,14 +555,14 @@ precsize_aton (char *cp, char **endptr)
 	int exponent;
 	int mantissa;
 
-	while (isdigit((int)*cp))
+	while (isdigit(*cp))
 		mval = mval * 10 + hexdigit_to_int(*cp++);
 
 	if (*cp == '.') {	/* centimeters */
 		cp++;
-		if (isdigit((int)*cp)) {
+		if (isdigit(*cp)) {
 			cmval = hexdigit_to_int(*cp++) * 10;
-			if (isdigit((int)*cp)) {
+			if (isdigit(*cp)) {
 				cmval += hexdigit_to_int(*cp++);
 			}
 		}
@@ -714,13 +612,14 @@ zparser_conv_loc(region_type *region, char *str)
 	int deg, min, secs;	/* Secs is stored times 1000.  */
 	uint32_t lat = 0, lon = 0, alt = 0;
 	/* encoded defaults: version=0 sz=1m hp=10000m vp=10m */
-	uint8_t vszhpvp[4] = {0, 0x12, 0x16, 0x13};
+	uint8_t vszhpvp[4] = {0, 0x12, 0x16, 0x13}; 
 	char *start;
 	double d;
+			
 
 	for(;;) {
 		deg = min = secs = 0;
-
+		
 		/* Degrees */
 		if (*str == '\0') {
 			zc_error_prev_line("unexpected end of LOC data");
@@ -729,25 +628,25 @@ zparser_conv_loc(region_type *region, char *str)
 
 		if (!parse_int(str, &str, &deg, "degrees", 0, 180))
 			return NULL;
-		if (!isspace((int)*str)) {
+		if (!isspace(*str)) {
 			zc_error_prev_line("space expected after degrees");
 			return NULL;
 		}
 		++str;
-
+		
 		/* Minutes? */
-		if (isdigit((int)*str)) {
+		if (isdigit(*str)) {
 			if (!parse_int(str, &str, &min, "minutes", 0, 60))
 				return NULL;
-			if (!isspace((int)*str)) {
+			if (!isspace(*str)) {
 				zc_error_prev_line("space expected after minutes");
 				return NULL;
 			}
 			++str;
 		}
-
+		
 		/* Seconds? */
-		if (isdigit((int)*str)) {
+		if (isdigit(*str)) {
 			start = str;
 			if (!parse_int(str, &str, &i, "seconds", 0, 60)) {
 				return NULL;
@@ -757,11 +656,11 @@ zparser_conv_loc(region_type *region, char *str)
 				return NULL;
 			}
 
-			if (!isspace((int)*str)) {
+			if (!isspace(*str)) {
 				zc_error_prev_line("space expected after seconds");
 				return NULL;
 			}
-			/* No need for precision specifiers, it's a double */
+
 			if (sscanf(start, "%lf", &d) != 1) {
 				zc_error_prev_line("error parsing seconds");
 			}
@@ -773,7 +672,7 @@ zparser_conv_loc(region_type *region, char *str)
 			secs = (int) (d * 1000.0 + 0.5);
 			++str;
 		}
-
+		
 		switch(*str) {
 		case 'N':
 		case 'n':
@@ -796,11 +695,11 @@ zparser_conv_loc(region_type *region, char *str)
 			return NULL;
 		}
 		++str;
-
+		
 		if (lat != 0 && lon != 0)
 			break;
 
-		if (!isspace((int)*str)) {
+		if (!isspace(*str)) {
 			zc_error_prev_line("space expected after latitude/longitude");
 			return NULL;
 		}
@@ -813,7 +712,7 @@ zparser_conv_loc(region_type *region, char *str)
 		return NULL;
 	}
 
-	if (!isspace((int)*str)) {
+	if (!isspace(*str)) {
 		zc_error_prev_line("space expected before altitude");
 		return NULL;
 	}
@@ -827,7 +726,7 @@ zparser_conv_loc(region_type *region, char *str)
 	}
 
 	/* Meters of altitude... */
-	(void) strtol(str, &str, 10);
+	(void)strtol(str, &str, 10);
 	switch(*str) {
 	case ' ':
 	case '\0':
@@ -837,7 +736,7 @@ zparser_conv_loc(region_type *region, char *str)
 		if (!parse_int(str + 1, &str, &i, "altitude fraction", 0, 99)) {
 			return NULL;
 		}
-		if (!isspace((int)*str) && *str != '\0' && *str != 'm') {
+		if (!isspace(*str) && *str != '\0' && *str != 'm') {
 			zc_error_prev_line("altitude fraction must be a number");
 			return NULL;
 		}
@@ -846,25 +745,25 @@ zparser_conv_loc(region_type *region, char *str)
 		zc_error_prev_line("altitude must be expressed in meters");
 		return NULL;
 	}
-	if (!isspace((int)*str) && *str != '\0')
+	if (!isspace(*str) && *str != '\0')
 		++str;
 
 	if (sscanf(start, "%lf", &d) != 1) {
 		zc_error_prev_line("error parsing altitude");
 	}
-
+	
 	alt = (uint32_t) (10000000.0 + d * 100 + 0.5);
 
-	if (!isspace((int)*str) && *str != '\0') {
+	if (!isspace(*str) && *str != '\0') {
 		zc_error_prev_line("unexpected character after altitude");
 		return NULL;
 	}
 
 	/* Now parse size, horizontal precision and vertical precision if any */
-	for(i = 1; isspace((int)*str) && i <= 3; i++) {
+	for(i = 1; isspace(*str) && i <= 3; i++) {
 		vszhpvp[i] = precsize_aton(str + 1, &str);
 
-		if (!isspace((int)*str) && *str != '\0') {
+		if (!isspace(*str) && *str != '\0') {
 			zc_error_prev_line("invalid size or precision");
 			return NULL;
 		}
@@ -903,7 +802,7 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 	uint8_t *t;
 	char *end;
 	long p;
-
+	
 	if (!colon) {
 		zc_error("address family separator is missing");
 		return NULL;
@@ -915,7 +814,7 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 
 	*colon = '\0';
 	*slash = '\0';
-
+	
 	if (*str == '!') {
 		negated = 1;
 		++str;
@@ -949,7 +848,7 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 	while (length > 0 && address[length - 1] == 0)
 		--length;
 
-
+	
 	p = strtol(slash + 1, &end, 10);
 	if (p < 0 || p > maximum_prefix) {
 		zc_error("prefix not in the range 0 .. %d", maximum_prefix);
@@ -964,7 +863,7 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 		    + length);
 	r = alloc_rdata(region, rdlength);
 	t = (uint8_t *) (r + 1);
-
+	
 	memcpy(t, &address_family, sizeof(address_family));
 	t += sizeof(address_family);
 	memcpy(t, &prefix, sizeof(prefix));
@@ -979,28 +878,28 @@ zparser_conv_apl_rdata(region_type *region, char *str)
 	return r;
 }
 
-/*
+/* 
  * Below some function that also convert but not to wireformat
  * but to "normal" (int,long,char) types
  */
 
-uint32_t
-zparser_ttl2int(const char *ttlstr, int* error)
+int32_t
+zparser_ttl2int(const char *ttlstr)
 {
 	/* convert a ttl value to a integer
 	 * return the ttl in a int
 	 * -1 on error
 	 */
 
-	uint32_t ttl;
+	int32_t ttl;
 	const char *t;
 
 	ttl = strtottl(ttlstr, &t);
 	if (*t != 0) {
 		zc_error_prev_line("invalid TTL value: %s",ttlstr);
-		*error = 1;
+		ttl = -1;
 	}
-
+    
 	return ttl;
 }
 
@@ -1008,7 +907,7 @@ zparser_ttl2int(const char *ttlstr, int* error)
 void
 zadd_rdata_wireformat(uint16_t *data)
 {
-	if (parser->current_rr.rdata_count >= MAXRDATALEN) {
+	if (parser->current_rr.rdata_count > MAXRDATALEN) {
 		zc_error_prev_line("too many rdata elements");
 	} else {
 		parser->current_rr.rdatas[parser->current_rr.rdata_count].data
@@ -1017,62 +916,10 @@ zadd_rdata_wireformat(uint16_t *data)
 	}
 }
 
-/**
- * Used for TXT RR's to grow with undefined number of strings.
- */
-void
-zadd_rdata_txt_wireformat(uint16_t *data, int first)
-{
-	rdata_atom_type *rd;
-	
-	/* First STR in str_seq, allocate 65K in first unused rdata
-	 * else find last used rdata */
-	if (first) {
-		rd = &parser->current_rr.rdatas[parser->current_rr.rdata_count];
-		if ((rd->data = (uint16_t *) region_alloc(parser->rr_region,
-			sizeof(uint16_t) + 65535 * sizeof(uint8_t))) == NULL) {
-			zc_error_prev_line("Could not allocate memory for TXT RR");
-			return;
-		}
-		parser->current_rr.rdata_count++;
-		rd->data[0] = 0;
-	}
-	else
-		rd = &parser->current_rr.rdatas[parser->current_rr.rdata_count-1];
-	
-	if ((size_t)rd->data[0] + (size_t)data[0] > 65535) {
-		zc_error_prev_line("too large rdata element");
-		return;
-	}
-	
-	memcpy((uint8_t *)rd->data + 2 + rd->data[0], data + 1, data[0]);
-	rd->data[0] += data[0];
-}
-
-/**
- * Clean up after last call of zadd_rdata_txt_wireformat
- */
-void
-zadd_rdata_txt_clean_wireformat()
-{
-	uint16_t *tmp_data;
-	rdata_atom_type *rd = &parser->current_rr.rdatas[parser->current_rr.rdata_count-1];
-	if ((tmp_data = (uint16_t *) region_alloc(parser->region, 
-		rd->data[0] + 2)) != NULL) {
-		memcpy(tmp_data, rd->data, rd->data[0] + 2);
-		rd->data = tmp_data;
-	}
-	else {
-		/* We could not get memory in non-volatile region */
-		zc_error_prev_line("could not allocate memory for rdata");
-		return;
-	}
-}
-
 void
 zadd_rdata_domain(domain_type *domain)
 {
-	if (parser->current_rr.rdata_count >= MAXRDATALEN) {
+	if (parser->current_rr.rdata_count > MAXRDATALEN) {
 		zc_error_prev_line("too many rdata elements");
 	} else {
 		parser->current_rr.rdatas[parser->current_rr.rdata_count].domain
@@ -1107,7 +954,7 @@ parse_unknown_rdata(uint16_t type, uint16_t *wireformat)
 		zc_error_prev_line("bad unknown RDATA");
 		return;
 	}
-
+	
 	for (i = 0; i < rdata_count; ++i) {
 		if (rdata_atom_is_domain(type, i)) {
 			zadd_rdata_domain(rdatas[i].domain);
@@ -1131,10 +978,10 @@ static int
 zrdatacmp(uint16_t type, rr_type *a, rr_type *b)
 {
 	int i = 0;
-
+	
 	assert(a);
 	assert(b);
-
+	
 	/* One is shorter than another */
 	if (a->rdata_count != b->rdata_count)
 		return 1;
@@ -1198,7 +1045,7 @@ zone_open(const char *filename, uint32_t ttl, uint16_t klass,
 }
 
 
-void
+void 
 set_bitnsec(uint8_t bits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE],
 	    uint16_t index)
 {
@@ -1208,7 +1055,7 @@ set_bitnsec(uint8_t bits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE],
 	 */
 	uint8_t window = index / 256;
 	uint8_t bit = index % 256;
-
+		
 	bits[window][bit / 8] |= (1 << (7 - bit % 8));
 }
 
@@ -1232,13 +1079,10 @@ process_rr(void)
 	int i;
 	rrtype_descriptor_type *descriptor
 		= rrtype_descriptor_by_type(rr->type);
-
+	
 	/* We only support IN class */
 	if (rr->klass != CLASS_IN) {
-		if(zone->opts && zone->opts->request_xfr)
-			zc_warning_prev_line("only class IN is supported");
-		else
-			zc_error_prev_line("only class IN is supported");
+		zc_error_prev_line("only class IN is supported");
 		return 0;
 	}
 
@@ -1250,40 +1094,38 @@ process_rr(void)
 		zc_error_prev_line("maximum rdata length exceeds %d octets", MAX_RDLENGTH);
 		return 0;
 	}
-
-	/* Do we have the zone already? */
-	if (!zone)
-	{
-		zone = (zone_type *) region_alloc(parser->region,
-							  sizeof(zone_type));
-		zone->apex = parser->default_apex;
-		zone->soa_rrset = NULL;
-		zone->soa_nx_rrset = NULL;
-		zone->ns_rrset = NULL;
-		zone->opts = NULL;
-		zone->is_secure = 0;
-		zone->updated = 1;
-
-		zone->next = parser->db->zones;
-		parser->db->zones = zone;
-		parser->current_zone = zone;
-	}
-
+		     
 	if (rr->type == TYPE_SOA) {
 		/*
 		 * This is a SOA record, start a new zone or continue
 		 * an existing one.
 		 */
 		if (rr->owner->is_apex) {
-			if(zone->opts && zone->opts->request_xfr)
-				zc_warning_prev_line("this SOA record was already encountered");
-			else
-				zc_error_prev_line("this SOA record was already encountered");
-		} else if (rr->owner == parser->default_apex) {
+			/*
+			   should be error!
+			   every zone should be passed to zonec only once.
+			*/
+			zone = namedb_find_zone(parser->db, rr->owner);
+			assert(zone);
+		} else {
+			/* new zone part */
+			zone = (zone_type *) region_alloc(parser->region,
+							  sizeof(zone_type));
 			zone->apex = rr->owner;
+			zone->soa_rrset = NULL;
+			zone->soa_nx_rrset = NULL;
+			zone->ns_rrset = NULL;
+			zone->opts = NULL;
+			zone->is_secure = 0;
+			zone->updated = 1;
+
+			/* insert in front of zone list */
+			zone->next = parser->db->zones;
+			parser->db->zones = zone;
+
 			rr->owner->is_apex = 1;
 		}
-
+		
 		/* parser part */
 		parser->current_zone = zone;
 	}
@@ -1291,10 +1133,7 @@ process_rr(void)
 	if (!dname_is_subdomain(domain_dname(rr->owner),
 				domain_dname(zone->apex)))
 	{
-		if(zone->opts && zone->opts->request_xfr)
-			zc_warning_prev_line("out of zone data");
-		else
-			zc_error_prev_line("out of zone data");
+		zc_error_prev_line("out of zone data");
 		return 0;
 	}
 
@@ -1307,7 +1146,7 @@ process_rr(void)
 		rrset->rr_count = 1;
 		rrset->rrs = (rr_type *) xalloc(sizeof(rr_type));
 		rrset->rrs[0] = *rr;
-
+			
 		region_add_cleanup(parser->region, cleanup_rrset, rrset);
 
 		/* Add it */
@@ -1339,52 +1178,40 @@ process_rr(void)
 	}
 
 	if(rr->type == TYPE_DNAME && rrset->rr_count > 1) {
-		if(zone->opts && zone->opts->request_xfr)
-			zc_warning_prev_line("multiple DNAMEs at the same name");
-		else
-			zc_error_prev_line("multiple DNAMEs at the same name");
+		zc_error_prev_line("multiple DNAMEs at the same name");
 	}
 	if(rr->type == TYPE_CNAME && rrset->rr_count > 1) {
-		if(zone->opts && zone->opts->request_xfr)
-			zc_warning_prev_line("multiple CNAMEs at the same name");
-		else
-			zc_error_prev_line("multiple CNAMEs at the same name");
+		zc_error_prev_line("multiple CNAMEs at the same name");
 	}
 	if((rr->type == TYPE_DNAME && domain_find_rrset(rr->owner, zone, TYPE_CNAME))
 	 ||(rr->type == TYPE_CNAME && domain_find_rrset(rr->owner, zone, TYPE_DNAME))) {
-		if(zone->opts && zone->opts->request_xfr)
-			zc_warning_prev_line("DNAME and CNAME at the same name");
-		else
-			zc_error_prev_line("DNAME and CNAME at the same name");
+		zc_error_prev_line("DNAME and CNAME at the same name");
 	}
 	if(domain_find_rrset(rr->owner, zone, TYPE_CNAME) &&
 		domain_find_non_cname_rrset(rr->owner, zone)) {
-		if(zone->opts && zone->opts->request_xfr)
-			zc_warning_prev_line("CNAME and other data at the same name");
-		else
-			zc_error_prev_line("CNAME and other data at the same name");
+		zc_error_prev_line("CNAME and other data at the same name");
 	}
 
-	if (rr->type == TYPE_RRSIG && rr_rrsig_type_covered(rr) == TYPE_DNSKEY) {
+#ifdef DNSSEC
+	if (rr->type == TYPE_RRSIG && rr_rrsig_type_covered(rr) == TYPE_SOA) {
 		rrset->zone->is_secure = 1;
 	}
-
+#endif
+	
 	/* Check we have SOA */
+	/* [XXX] this is dead code */
 	if (zone->soa_rrset == NULL) {
-		if (rr->type == TYPE_SOA) {
-			if (rr->owner != zone->apex) {
-				zc_error_prev_line(
-					"SOA record with invalid domain name");
-			} else {
-				zone->soa_rrset = rrset;
-			}
+		if (rr->type != TYPE_SOA) {
+			zc_error_prev_line(
+				"missing SOA record on top of the zone");
+		} else if (rr->owner != zone->apex) {
+			zc_error_prev_line(
+				"SOA record with invalid domain name");
+		} else {
+			zone->soa_rrset = rrset;
 		}
-	}
-	else if (rr->type == TYPE_SOA) {
-		if(zone->opts && zone->opts->request_xfr)
-			zc_warning_prev_line("duplicate SOA record discarded");
-		else
-			zc_error_prev_line("duplicate SOA record discarded");
+	} else if (rr->type == TYPE_SOA) {
+		zc_error_prev_line("duplicate SOA record discarded");
 		--rrset->rr_count;
 	}
 
@@ -1393,7 +1220,7 @@ process_rr(void)
 		zone->ns_rrset = rrset;
 	}
 	if (vflag > 1 && totalrrs > 0 && (totalrrs % progress == 0)) {
-		fprintf(stdout, "%ld\n", totalrrs);
+		printf("%ld\n", totalrrs);
 	}
 	++totalrrs;
 	return 1;
@@ -1409,16 +1236,16 @@ domain_find_rrset_any(domain_type *domain, uint16_t type)
 	while (result) {
 		if (rrset_rrtype(result) == type) {
 			return result;
-		}
+                }
 		result = result->next;
-	}
-	return NULL;
+        }
+        return NULL;
 }
 
 /*
  * Check for DNAME type. Nothing is allowed below it
  */
-static void
+static void 
 check_dname(namedb_type* db)
 {
 	domain_type* domain;
@@ -1461,7 +1288,7 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 		zc_error("incorrect zone name '%s'", name);
 		return;
 	}
-
+	
 #ifndef ROOT_SERVER
 	/* Is it a root zone? Are we a root server then? Idiot proof. */
 	if (dname->label_count == 1) {
@@ -1477,7 +1304,7 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 			zone_options_t* zopt = zone_options_find(nsd_options, dname);
 			if(zopt && zone_is_slave(zopt)) {
 				zc_warning("slave zone %s with no zonefile '%s'(%s) will "
-					"force zone transfer.",
+					"force zone transfer.", 
 					name, zonefile, strerror(errno));
 				return;
 			}
@@ -1491,7 +1318,7 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 	yyparse();
 
 	/* check if zone file contained a correct SOA record */
-	if (parser->current_zone && parser->current_zone->soa_rrset
+	if (parser->current_zone && parser->current_zone->soa_rrset 
 		&& parser->current_zone->soa_rrset->rr_count!=0)
 	{
 		if(dname_compare(domain_dname(
@@ -1505,13 +1332,13 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 	}
 
 	fclose(yyin);
-
+	
 	fflush(stdout);
 	totalerrors += parser->errors;
 	parser->filename = NULL;
 }
 
-static void
+static void 
 usage (void)
 {
 #ifndef NDEBUG
@@ -1520,7 +1347,7 @@ usage (void)
 	fprintf(stderr, "usage: zonec [-v|-h|-C] [-c configfile] [-o origin] [-d directory] [-f database] [-z zonefile]\n\n");
 #endif
 	fprintf(stderr, "\tNSD zone compiler, creates database from zone files.\n");
-	fprintf(stderr, "\tVersion %s. Report bugs to <%s>.\n\n",
+	fprintf(stderr, "\tVersion %s. Report bugs to <%s>.\n\n", 
 		PACKAGE_VERSION, PACKAGE_BUGREPORT);
 	fprintf(stderr, "\t-v\tBe more verbose.\n");
 	fprintf(stderr, "\t-h\tPrint this help information.\n");
@@ -1539,7 +1366,7 @@ usage (void)
 extern char *optarg;
 extern int optind;
 
-int
+int 
 main (int argc, char **argv)
 {
 	struct namedb *db;
@@ -1551,7 +1378,7 @@ main (int argc, char **argv)
 	const char* zonesdir = NULL;
 	const char* singlefile = NULL;
 	nsd_options_t* nsd_options = NULL;
-
+	
 	log_init("zonec");
 
 	global_region = region_create(xalloc, free);
@@ -1631,16 +1458,11 @@ main (int argc, char **argv)
 
 	/* Create the database */
 	if ((db = namedb_new(dbfile)) == NULL) {
-		fprintf(stderr, "zonec: error creating the database (%s): %s\n",
-			dbfile, strerror(errno));
+		fprintf(stderr, "zonec: error creating the database: %s\n", dbfile);
 		exit(1);
 	}
 
 	parser = zparser_create(global_region, rr_region, db);
-	if (!parser) {
-		fprintf(stderr, "zonec: error creating the parser\n");
-		exit(1);
-	}
 
 	/* Unique pointers used to mark errors.	 */
 	error_dname = (dname_type *) region_alloc(global_region, 0);
@@ -1650,7 +1472,7 @@ main (int argc, char **argv)
 		/*
 		 * Read a single zone file with the specified origin
 		 */
-		if(!singlefile) {
+		if(!singlefile || !origin) {
 			fprintf(stderr, "zonec: must have -z zonefile when reading single zone.\n");
 			exit(1);
 		}
@@ -1659,10 +1481,10 @@ main (int argc, char **argv)
 			exit(1);
 		}
 		if (vflag > 0)
-			fprintf(stdout, "zonec: reading zone \"%s\".\n", origin);
+			fprintf(stderr, "zonec: reading zone \"%s\".\n", origin);
 		zone_read(origin, singlefile, nsd_options);
 		if (vflag > 0)
-			fprintf(stdout, "zonec: processed %ld RRs in \"%s\".\n", totalrrs, origin);
+			fprintf(stderr, "zonec: processed %ld RRs in \"%s\".\n", totalrrs, origin);
 	} else {
 		zone_options_t* zone;
 		if(!nsd_options) {
@@ -1673,11 +1495,11 @@ main (int argc, char **argv)
 		RBTREE_FOR(zone, zone_options_t*, nsd_options->zone_options)
 		{
 			if (vflag > 0)
-				fprintf(stdout, "zonec: reading zone \"%s\".\n",
+				fprintf(stderr, "zonec: reading zone \"%s\".\n",
 					zone->name);
 			zone_read(zone->name, zone->zonefile, nsd_options);
 			if (vflag > 0)
-				fprintf(stdout,
+				fprintf(stderr,
 					"zonec: processed %ld RRs in \"%s\".\n",
 					totalrrs, zone->name);
 			totalrrs = 0;
@@ -1686,37 +1508,31 @@ main (int argc, char **argv)
 	check_dname(db);
 
 #ifndef NDEBUG
-	if (vflag > 0) {
-		fprintf(stdout, "global_region: ");
-		region_dump_stats(global_region, stdout);
-		fprintf(stdout, "\n");
-		fprintf(stdout, "db->region: ");
-		region_dump_stats(db->region, stdout);
-		fprintf(stdout, "\n");
-	}
+	fprintf(stderr, "global_region: ");
+	region_dump_stats(global_region, stderr);
+	fprintf(stderr, "\n");
+	fprintf(stderr, "db->region: ");
+	region_dump_stats(db->region, stderr);
+	fprintf(stderr, "\n");
 #endif /* NDEBUG */
-
+	
 	/* Close the database */
 	if (namedb_save(db) != 0) {
-		fprintf(stderr, "zonec: error writing the database (%s): %s\n", db->filename, strerror(errno));
+		fprintf(stderr, "zonec: error saving the database: %s\n", strerror(errno));
 		namedb_discard(db);
 		exit(1);
 	}
 
 	/* Print the total number of errors */
 	if (vflag > 0 || totalerrors > 0) {
-		if (totalerrors > 0) {
-			fprintf(stderr, "\nzonec: done with %ld errors.\n",
+		fprintf(stderr, "\nzonec: done with %ld errors.\n",
 			totalerrors);
-		} else {
-			fprintf(stdout, "\nzonec: done with no errors.\n");
-		}
 	}
-
+	
 	/* Disable this to save some time.  */
 #if 0
 	region_destroy(global_region);
 #endif
-
+	
 	return totalerrors ? 1 : 0;
 }
