@@ -1,7 +1,9 @@
-/* A Bison parser, made from zparser.y, by GNU bison 1.75.  */
+/* A Bison parser, made by GNU Bison 2.3.  */
 
-/* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.
+/* Skeleton interface for Bison's Yacc-like parsers in C
+
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,16 +17,21 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
-/* As a special exception, when this file is copied by Bison into a
-   Bison output file, you may use that output file without restriction.
-   This special exception was added by the Free Software Foundation
-   in version 1.24 of Bison.  */
+/* As a special exception, you may create a larger work that contains
+   part or all of the Bison parser skeleton and distribute that work
+   under terms of your choice, so long as that work isn't itself a
+   parser generator using the skeleton or a modified version thereof
+   as a parser skeleton.  Alternatively, if you modify or redistribute
+   the parser skeleton itself, you may (at your option) remove this
+   special exception, which will cause the skeleton and the resulting
+   Bison output files to be licensed under the GNU General Public
+   License without this special exception.
 
-#ifndef BISON_ZPARSER_H
-# define BISON_ZPARSER_H
+   This special exception was added by the Free Software Foundation in
+   version 2.2 of Bison.  */
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -107,6 +114,7 @@
      T_UTYPE = 330
    };
 #endif
+/* Tokens.  */
 #define T_A 258
 #define T_NS 259
 #define T_MX 260
@@ -184,9 +192,10 @@
 
 
 
-#ifndef YYSTYPE
+#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
+typedef union YYSTYPE
 #line 49 "zparser.y"
-typedef union {
+{
 	domain_type	 *domain;
 	const dname_type *dname;
 	struct lex_data	  data;
@@ -194,14 +203,14 @@ typedef union {
 	uint16_t	  klass;
 	uint16_t	  type;
 	uint16_t	 *unknown;
-} yystype;
-/* Line 1237 of /usr/local/share/bison/yacc.c.  */
-#line 200 "zparser.h"
-# define YYSTYPE yystype
+}
+/* Line 1489 of yacc.c.  */
+#line 209 "zparser.h"
+	YYSTYPE;
+# define yystype YYSTYPE /* obsolescent; will be withdrawn */
+# define YYSTYPE_IS_DECLARED 1
+# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 extern YYSTYPE yylval;
 
-
-#endif /* not BISON_ZPARSER_H */
-
